#!/bin/sh

# =============================================================================

: << END-OF-MANUAL-PAGE
// = AUTHOR(S)
//     Graham Dumpleton
// 
// = COPYRIGHT
//     Copyright 1991 OTC LIMITED
//     Copyright 1995-1997 DUMPLETON SOFTWARE CONSULTING PTY LIMITED

// [class2html]
//
// = TITLE
//     Generates HTML manual pages from input files.
//
// = SYNOPSIS
//     class2html [ -ext html ] file.hh
//     class2html [ -nested ] file.hh
//     class2html [ -link 'path/*.html' ] file.ci
//
// = DESCRIPTION
//     The <class2html> program generates a HTML style manual page from
//     an input file. The names of the generated files will correspond to the
//     names of the classes the classinfo file describes. The <class2html>
//     program achieves this by running <class2info> on the input file and
//     then running <info2html> on the generated classinfo file. Options to
//     this program are simply passed onto <info2html>.
//
//     By default, manual pages for nested classes will not be output.
//     To generated manual pages for nested classes specify the "-nested"
//     option.
//
//     By default the suffix appended to the manual pages will be ".html". The
//     extension appended to the manual pages can be changed by specifying
//     the "-ext" option.
//
//     By default, no cross references are inserted into generated file.
//     Cross references can be generated by supplying "-link" with the
//     argument being a glob style pattern of the files which can be linked
//     to. Note that the files must exist, so this program might need to
//     be run twice, the first time to generate the files, the second
//     time so that links can be put in.
//
// = NOTES
//     Although reference to generating manual pages for only classes is
//     mentioned, manual pages for structs, unions and templates are
//     also generated. Further, if the syntax for standalone manual
//     pages has been used, then manual pages will be generated for that
//     also
//
// = SEE ALSO
//     class2info(1), info2html(1)
END-OF-MANUAL-PAGE

# =============================================================================

OSE_HOST=@OSE_HOST@

OSE_RELEASE_DIRECTORY=@OSE_RELEASE_DIRECTORY@
export OSE_RELEASE_DIRECTORY

OSE_ROOT=${OSE_ROOT-@OSE_ROOT@}
export OSE_ROOT

OSE_VERSION_ROOT=$OSE_ROOT/$OSE_RELEASE_DIRECTORY
export OSE_VERSION_ROOT

BINDIR="$OSE_VERSION_ROOT/$OSE_HOST/bin"
LIBDIR=${CLASSINFOLIBDIR-"$OSE_VERSION_ROOT/lib"}

CLASS2INFO="$BINDIR/class2info@CMDEXT@"
INFO2HTML="$BINDIR/info2html@CMDEXT@"

FILES=
MANEXT="html"
PATTERNS=
NESTED=

#
# Error.
#
ERROR()
{
  echo "`basename $0`: $1" >&2
  shift
  while test $# != "0"
  do
    echo $1 >&2
    shift
  done
  exit 1
}

#
# Usage message.
#
USAGE()
{
  ERROR "Usage: `basename $0` files"
} 

#
# Parse command line.
#
while test $# != 0
do
  case $1 in
    -link)
      shift
      if test "$#" != "0"
      then
        PATTERNS=$1
        shift
      fi
      ;;
    -ext)
      shift
      if test "$#" != "0"
      then
        MANEXT=$1
        shift
      fi
      ;;
    -nested)
      shift
      NESTED=1
      ;;
    *)
      FILES="$FILES $1"
      shift
      ;;
    esac
done

#
# Check usage.
#
if test "$FILES" = ""
then
  USAGE
fi

for input in $FILES
do
  $CLASS2INFO $input
  if test "$?" != "0"
  then
    exit 1
  fi
  file=`basename $input`
  base="`echo $file | sed -e 's/\..*$//'`"
  if test -f "$base.ci"
  then
    if test "$NESTED" = "1"
    then
      if test "$PATTERNS" = ""
      then
	$INFO2HTML -ext "$MANEXT" -nested $base.ci
      else
	$INFO2HTML -link "$PATTERNS" -ext "$MANEXT" -nested $base.ci
      fi
    else
      if test "$PATTERNS" = ""
      then
	$INFO2HTML -ext "$MANEXT" $base.ci
      else
	$INFO2HTML -link "$PATTERNS" -ext "$MANEXT" $base.ci
      fi
    fi
    rm -f $base.ci
  fi
done
