#ifndef OTC_DISPATCH_EVCLLBCK_HH
#define OTC_DISPATCH_EVCLLBCK_HH
/*
// ============================================================================
//
// = LIBRARY
//     OTC
// 
// = FILENAME
//     dispatch/evcllbck.hh
//
// = AUTHOR(S)
//     Graham Dumpleton
// 
// = COPYRIGHT
//     Copyright 1997-1999 DUMPLETON SOFTWARE CONSULTING PTY LIMITED
//
// ============================================================================
*/

#include <OTC/memory/cmmnpool.hh>

#ifdef __GNUG__
#pragma interface "OTC/dispatch/evcllbck.hh"
#endif

/* ------------------------------------------------------------------------- */

class OTC_Event;

class OSE_EXPORT OTC_EVCallback
    // = TITLE
    //     Base class for implementing callback objects for events.
    //
    // = CLASS TYPE
    //     Abstract
    //
    // = DESCRIPTION
    //     The <OTC_EVCallback> class is used as a base class for
    //     implementing callback objects to be used in conjunction with
    //     <OTC_EVThreadPool>. The derived class might hold a pointer to
    //     a function to execute, passing the event as argument, or it may
    //     actually contain an implementation which directly deals with the
    //     event in some way.
    //     
    // = SEE ALSO
    //     <OTC_EVThreadPool>, <OTC_EVHandler>
{
  public:

    virtual             ~OTC_EVCallback();

#if !defined(ENV_NOCOMMONPOOL)
    void*               operator new(size_t theSize)
                                { return OTC_CommonPool::allocate(theSize); }

    void                operator delete(void* theMemory, size_t theSize)
                                { OTC_CommonPool::release(theMemory,theSize); }
#endif

    // = EXECUTION

    virtual void	execute(OTC_Event* theEvent) = 0;
				// Must be redefined in a derived class to
				// perform the actual delivery of <theEvent>
				// to a callback function.

    // = DESTRUCTION

    virtual void	destroy();
				// Generally called once the callback has
				// been executed. The default action is to
				// delete the object. This can be overridden
				// in a derived class if necessary.

  protected:

			OTC_EVCallback() {}

  private:

			OTC_EVCallback(OTC_EVCallback const&);
				// Do not define an implementation for this.

    OTC_EVCallback&	operator=(OTC_EVCallback const&);
				// Do not define an implementation for this.
};

/* ------------------------------------------------------------------------- */

#endif /* OTC_DISPATCH_EVCLLBCK_HH */
