/*
// ============================================================================
//
// = LIBRARY
//     OTC
// 
// = FILENAME
//     debug/otclogbuf.cc
//
// = AUTHOR(S)
//     Graham Dumpleton
// 
// = COPYRIGHT
//     Copyright 1993 OTC LIMITED
//     Copyright 1995-2000 DUMPLETON SOFTWARE CONSULTING PTY LIMITED
//
// ============================================================================
*/

#ifdef __GNUG__
#pragma implementation "OTC/debug/logbuf.hh"
#endif

#include <OTC/debug/logbuf.hh>

#ifndef zapeof
#define zapeof(c) ((c)&0377)
#endif

#if defined(ENV_OSTORE) && defined(CXX_DEC) && defined(__alpha)
#pragma __environment save
#pragma __environment header_defaults
#endif

/* ------------------------------------------------------------------------- */
OTC_LogBuf::~OTC_LogBuf()
{
  // Nothing to do.
}

/* ------------------------------------------------------------------------- */
OTC_LogBuf::OTC_LogBuf(char* theBuffer, int theSize)
  : channel_(0), level_(OTCLIB_LOG_DEBUG)
{
  if (theBuffer != 0 && theSize > (int)sizeof(buffer_))
    setp(theBuffer,theBuffer+theSize-2);
  else
    setp(buffer_,buffer_+sizeof(buffer_)-2);
}

/* ------------------------------------------------------------------------- */
#if defined(ENV_CXXSTDLIB)
OTC_LogBuf::int_type OTC_LogBuf::overflow(int_type c)
#else
int OTC_LogBuf::overflow(int c)
#endif
{
  char* endp = pptr();

  *endp = EOS;
  *(endp+1) = EOS;

  while (*(endp-1) == '\n' && endp > pbase())
    *--endp = EOS;

#if defined(ENV_CXXSTDLIB)
  if (!traits_type::eq_int_type(c,traits_type::eof()) && c != EOS)
#else
  if (c != EOF && c != EOS)
#endif
    *endp++ = char(c);

  OTC_Logger::notify(channel_,level_,pbase(),endp-pbase());

  setp(pbase(),epptr());

#if defined(ENV_CXXSTDLIB)
  return traits_type::not_eof(c);
#else
  return zapeof(c);
#endif
}

/* ------------------------------------------------------------------------- */
#if defined(ENV_CXXSTDLIB)
OTC_LogBuf::int_type OTC_LogBuf::underflow()
#else
int OTC_LogBuf::underflow()
#endif
{
  return 0;
}

/* ------------------------------------------------------------------------- */
int OTC_LogBuf::sync()
{
  if (pptr() && pptr() > pbase())
  {
    // LIBG++ does check explicitly for -1, so must make sure
    // we return 0 if all is okay.

#if defined(ENV_CXXSTDLIB)
    int_type c = overflow(traits_type::eof());
    if (traits_type::eq_int_type(c,traits_type::eof()))
      return -1;
    else
      return 0;
#else
    int c = overflow(EOF);
    return c == -1 ? -1 : 0;
#endif
  }

  return 0;
}

#if defined(ENV_OSTORE) && defined(CXX_DEC) && defined(__alpha)
#pragma __environment restore
#endif

/* ------------------------------------------------------------------------- */
