/*
// ============================================================================
//
// = LIBRARY
//     OTC
// 
// = FILENAME
//     dispatch/_signal.cc
//
// = AUTHOR(S)
//     Graham Dumpleton
// 
// = COPYRIGHT
//     Copyright 1993 OTC LIMITED
//     Copyright 1996-1999 DUMPLETON SOFTWARE CONSULTING PTY LIMITED
//
// ============================================================================
*/

#include <OTC/dispatch/signal.hh>
#include <OTC/dispatch/evagent.hh>
#include <OTC/dispatch/dispatch.hh>

#include <OTC/program/program.hh>
#include <OTC/debug/tracer.hh>
#include <OTC/debug/logstrm.hh>

#include <stdlib.h>
#include <signal.h>
#if defined(HAVE_UNISTD_H)
#include <unistd.h>
#endif

#ifdef _MSC_VER
#include <windows.h>
inline void sleep(int period) { Sleep(period*1000); }
#endif

/* ------------------------------------------------------------------------- */

class Agent1 : public OTC_EVAgent
{
  public:

			Agent1()
			  : count_(0)
				{}

  protected:

    void		handle(OTC_Event* theEvent);

  private:

    int			count_;
};

void Agent1::handle(OTC_Event* theEvent)
{
  OTC_Tracer tracer("Agent1::handle()");

  ostream& outs = tracer();
  theEvent->dump(outs);
  outs << endl;
  theEvent->destroy();

  count_++;

  if (count_ == 5)
    OTCEV_Signal::unsubscribeAgent(id());
}

int main(int argc, char* argv[])
{
  set_terminate(otclib_terminate_function);

  OTC_Program::initialise(argc,argv);

  OTC_Tracer tracer("main()");

  OTC_Dispatcher::initialise();

  Agent1 agent;

  OTCEV_Signal::capacity(4);

  OTCEV_Signal::subscribe(agent.id(),SIGINT);
#ifdef SIGUSR1
  OTCEV_Signal::subscribe(agent.id(),SIGUSR1);
#endif
#ifdef SIGUSR2
  OTCEV_Signal::subscribe(agent.id(),SIGUSR2);
#endif

  OTCEV_Signal::unsubscribeAgent(agent.id());

  OTCEV_Signal::subscribe(agent.id(),SIGINT);
#ifdef SIGUSR1
  OTCEV_Signal::subscribe(agent.id(),SIGUSR1);
#endif
#ifdef SIGUSR2
  OTCEV_Signal::subscribe(agent.id(),SIGUSR2);
#endif

#if 0
  while (1)
  {
    tracer() << "sleeping" << endl;

    sleep(2);

    tracer() << "look for new jobs" << endl;

    OTC_Job* job;
    job = OTCEV_Signal::pending();

    if (job != 0)
    {
      tracer() << "execute job" << endl;

      job->execute();
      delete job;
    }
  }
#endif

  OTC_Dispatcher::run();

  return 0;
}

/* ------------------------------------------------------------------------- */
