/*
// ============================================================================
//
// = LIBRARY
//     OTC
// 
// = FILENAME
//     misc/otcfuncbuf.cc
//
// = AUTHOR(S)
//     Graham Dumpleton
// 
// = COPYRIGHT
//     Copyright 1997-2000 DUMPLETON SOFTWARE CONSULTING PTY LIMITED
//
// ============================================================================
*/

#ifdef __GNUG__
#pragma implementation "OTC/misc/funcbuf.hh"
#endif

#include <OTC/misc/funcbuf.hh>

#ifndef zapeof
#define zapeof(c) ((c)&0377)
#endif

#if defined(ENV_OSTORE) && defined(CXX_DEC) && defined(__alpha)
#pragma __environment save
#pragma __environment header_defaults
#endif

/* ------------------------------------------------------------------------- */
OTC_FuncBuf::~OTC_FuncBuf()
{
  // Nothing to do.
}

/* ------------------------------------------------------------------------- */
OTC_FuncBuf::OTC_FuncBuf(
 void (*theFunc)(char const*,u_int),
 char* theBuffer,
 u_int theSize
)
  : func_(theFunc)
{
  if (theBuffer != 0 && theSize > (int)sizeof(buffer_))
    setp(theBuffer,theBuffer+theSize-2);
  else
    setp(buffer_,buffer_+sizeof(buffer_)-2);
}

/* ------------------------------------------------------------------------- */
#if defined(ENV_CXXSTDLIB)
OTC_FuncBuf::int_type OTC_FuncBuf::overflow(int_type c)
#else
int OTC_FuncBuf::overflow(int c)
#endif
{
  char* endp = pptr();

  *endp = '\0';
  *(endp+1) = '\0';

#if defined(ENV_CXXSTDLIB)
  if (!traits_type::eq_int_type(c,traits_type::eof()) && c != '\0')
#else
  if (c != EOF && c != '\0')
#endif
    *endp++ = char(c);

  (*func_)(pbase(),endp-pbase());

  setp(pbase(),epptr());

#if defined(ENV_CXXSTDLIB)
  return traits_type::not_eof(c);
#else
  return zapeof(c);
#endif
}

/* ------------------------------------------------------------------------- */
#if defined(ENV_CXXSTDLIB)
OTC_FuncBuf::int_type OTC_FuncBuf::underflow()
#else
int OTC_FuncBuf::underflow()
#endif
{
  return 0;
}

/* ------------------------------------------------------------------------- */
int OTC_FuncBuf::sync()
{
  if (pptr() && pptr() > pbase())
  {
    // LIBG++ does check explicitly for -1, so must make sure
    // we return 0 if all is okay.

#if defined(ENV_CXXSTDLIB)
    int_type c = overflow(traits_type::eof());
    if (traits_type::eq_int_type(c,traits_type::eof()))
      return -1;
    else
      return 0;
#else
    int c = overflow(EOF);
    return c == -1 ? -1 : 0;
#endif
  }

  return 0;
}

#if defined(ENV_OSTORE) && defined(CXX_DEC) && defined(__alpha)
#pragma __environment restore
#endif

/* ------------------------------------------------------------------------- */
