/*
// ============================================================================
//
// = LIBRARY
//     OTC
//
// = FILENAME
//     refcnt/otcresource.cc
//
// = AUTHOR(S)
//     Graham Dumpleton
//
// = COPYRIGHT
//     Copyright 1991 OTC LIMITED
//     Copyright 1994-2000 DUMPLETON SOFTWARE CONSULTING PTY LIMITED
//
// ============================================================================
*/

#ifdef __GNUG__
#pragma implementation "OTC/refcnt/resource.hh"
#endif

#include <OTC/refcnt/resource.hh>
#include <OTC/memory/cmmnpool.hh>

#include <stdlib.h>

/* ------------------------------------------------------------------------- */
OTC_NRMutex OTC_Resource::mutex_;
OTC_Boolean OTC_Resource::globAllocatedUsingNew_ = OTCLIB_FALSE;
OTC_Boolean OTC_Resource::globInitialised_ = OTCLIB_FALSE;
OTC_Boolean OTC_Resource::globResourceWarnings_ = OTCLIB_FALSE;

/* ------------------------------------------------------------------------- */
OTC_Resource::OTC_Resource()
{
#if defined(ENV_OSTORE)
  OTC_Locality theLocality = OTC_Locality::of(this);
#endif

#if defined(ENV_OSTORE)
  refCount_ = new (theLocality,OTC_Count::get_os_typespec()) OTC_Count;
#else
  refCount_ = new OTC_Count;
#endif
  OTCLIB_ASSERT_M(refCount_ != 0);

#if !defined(ENV_OSTORE) && !defined(ENV_THREADS)

  if (globInitialised_ == OTCLIB_FALSE)
  {
    globInitialised_ = OTCLIB_TRUE;
    char* env = getenv("OTCLIB_RESOURCEWARNINGS");
    if (env != 0)
      globResourceWarnings_ = OTCLIB_TRUE;
  }

  allocatedUsingNew_ = globAllocatedUsingNew_;
  globAllocatedUsingNew_ = OTCLIB_FALSE;
#endif
}

/* ------------------------------------------------------------------------- */
OTC_Resource::~OTC_Resource()
{
  OTCLIB_ENSURE_FN((numRefs() == 0),
   "OTC_Resource::~OTC_Resource()",
   "object still referenced");

  delete refCount_;
}

/* ------------------------------------------------------------------------- */
u_int OTC_Resource::numRefs() const
{
  u_int theResult;

  mutex_.lock();

  theResult = refCount_->count();

  mutex_.unlock();

  return theResult;
}

/* ------------------------------------------------------------------------- */
u_int OTC_Resource::reference() const
{
#if !defined(ENV_OSTORE) && !defined(ENV_THREADS)
  if (globResourceWarnings_ != OTCLIB_FALSE)
  {
    OTCLIB_ENSURE_FN_W((allocatedUsingNew_ != OTCLIB_FALSE),
     "OTC_Resource::reference()",
     "object not allocated using new");
  }
#endif

  u_int theResult;

  mutex_.lock();

  refCount_->inc();
  theResult = refCount_->count();

  mutex_.unlock();

  return theResult;
}

/* ------------------------------------------------------------------------- */
u_int OTC_Resource::unReference() const
{
#if !defined(ENV_OSTORE) && !defined(ENV_THREADS)
  if (globResourceWarnings_ != OTCLIB_FALSE)
  {
    if (allocatedUsingNew_ == OTCLIB_FALSE)
    {
      OTCLIB_ENSURE_FN_W((allocatedUsingNew_ != OTCLIB_FALSE),
       "OTC_Resource::unReference()",
       "object not allocated using new");
    }
  }
#endif

  OTCLIB_ENSURE_FN((numRefs() != 0),
   "OTC_Resource::unReference()",
   "object not referenced");

  u_int theResult;

  mutex_.lock();

  refCount_->dec();
  theResult = refCount_->count();

  mutex_.unlock();

  if (theResult == 0)
    delete (OTC_Resource*)this;

  return theResult;
}

#if !defined(ENV_OSTORE) && !defined(ENV_THREADS)
/* ------------------------------------------------------------------------- */
void* OTC_Resource::operator new(size_t theSize)
{
  doInOperatorNew();

#if defined(ENV_NOCOMMONPOOL)
  return ::operator new(theSize);
#else
  return OTC_MPObject::operator new(theSize);
#endif
}

/* ------------------------------------------------------------------------- */
void OTC_Resource::operator delete(void* theMem, size_t theSz)
{
#if defined(ENV_NOCOMMONPOOL)
  ::operator delete(theMem);
#else
  OTC_MPObject::operator delete(theMem,theSz);
#endif
}
#endif

/* ------------------------------------------------------------------------- */
