/*
// ============================================================================
//
// = LIBRARY
//     OTC
//
// = FILENAME
//     text/otcrstring.cc
//
// = AUTHOR(S)
//     Graham Dumpleton
//
// = COPYRIGHT
//     Copyright 1993-1994 TELSTRA CORPORATION LIMITED
//     Copyright 1994-2001 DUMPLETON SOFTWARE CONSULTING PTY LIMITED
//
// ============================================================================
*/

#ifdef __GNUG__
#pragma implementation "OTC/text/rstring.hh"
#endif

#include <OTC/text/rstring.hh>
#include <OTC/collctn/hashtble.hh>
#include <OTC/collctn/strgactn.hh>
#include <OTC/memory/cmmnpool.hh>
#include <OTC/thread/mxreaper.hh>

#include <OTC/stdlib/iostream.h>

#include <stdlib.h>

/* ------------------------------------------------------------------------- */
OTC_NRMutex OTC_RString::mutex_;
OTC_NRMutex OTC_RString::sdmutex_;
OTC_Boolean OTC_RString::initialised_ = OTCLIB_FALSE;
OTC_BufferingType OTC_RString::bufType_ = OTCLIB_BUFFERING1;
OTC_HashTable* OTC_RString::index_ = 0;

#if defined(ENV_OSTORE)
/* ------------------------------------------------------------------------- */
os_typespec* OTC_StringData::typespec()
{
  static os_typespec ts("OTC_StringData");
  return &ts;
}
#endif

/* ------------------------------------------------------------------------- */
OTC_StringData::~OTC_StringData()
{
  // Nothing to do.
}

/* ------------------------------------------------------------------------- */
OTC_RString::~OTC_RString()
{
  OTC_StringData* oldData = 0;

  sdmutex_.lock();

  OTC_MutexReaper<OTC_NRMutex> xxxMutex;
  xxxMutex.grab(sdmutex_);

  if (data_->count == 2 && data_->symbol != OTCLIB_FALSE)
  {
    int tmpHashValue;
    tmpHashValue = OTC_Hash::hash(data_->data,data_->length);
    index_->remove(data_,tmpHashValue);

    oldData = data_;
  }
  else if (data_->count == 1)
    oldData = data_;
  else
    data_->count--;

  xxxMutex.release();

  sdmutex_.unlock();

  if (oldData != 0)
  {
    release_(oldData->data,oldData->capacity);
    delete oldData;
  }
}

/* ------------------------------------------------------------------------- */
OTC_RString::OTC_RString(
 u_int theLength,
 OTC_StringBuffering theType
)
{
#if defined(ENV_OSTORE)
  OTC_Locality theLocality = OTC_Locality::of(this);
#endif

#if defined(ENV_OSTORE)
  data_ = new (theLocality,OTC_StringData::get_os_typespec()) OTC_StringData;
#else
  data_ = new OTC_StringData;
#endif
  OTCLIB_ASSERT_M(data_ != 0);

  data_->count = 1;
  data_->length = theLength;
  data_->capacity = data_->length+1;
  data_->symbol = OTCLIB_FALSE;

  if (theType == OTCLIB_BUFFERED)
  {
    data_->capacity = capacity_(data_->length)+1;
    OTCLIB_ASSERT(data_->capacity >= data_->length+1);
  }

#if defined(ENV_OSTORE)
  data_->data = new (theLocality,os_typespec::get_char(),
   data_->capacity) char[data_->capacity];
#else
  data_->data = allocate_(data_->capacity);
#endif
  OTCLIB_ASSERT_M(data_->data != 0);

  data_->data[data_->length] = EOS;
}

/* ------------------------------------------------------------------------- */
OTC_RString::OTC_RString(
 u_int theLength,
 void const* theObject,
 OTC_StringBuffering theType
)
{
#if defined(ENV_OSTORE)
  OTC_Locality theLocality;
  if (theObject != 0)
    theLocality = OTC_Locality::of(theObject);
  else
    theLocality = OTC_Locality::of(this);
#endif

#if defined(ENV_OSTORE)
  data_ = new (theLocality,OTC_StringData::get_os_typespec()) OTC_StringData;
#else
  data_ = new OTC_StringData;
#endif
  OTCLIB_ASSERT_M(data_ != 0);

  data_->count = 1;
  data_->length = theLength;
  data_->capacity = data_->length+1;
  data_->symbol = OTCLIB_FALSE;

  if (theType == OTCLIB_BUFFERED)
  {
    data_->capacity = capacity_(data_->length)+1;
    OTCLIB_ASSERT(data_->capacity >= data_->length+1);
  }

#if defined(ENV_OSTORE)
  data_->data = new (theLocality,os_typespec::get_char(),
   data_->capacity) char[data_->capacity];
#else
  data_->data = allocate_(data_->capacity);
#endif
  OTCLIB_ASSERT_M(data_->data != 0);

  data_->data[data_->length] = EOS;
}

/* ------------------------------------------------------------------------- */
OTC_RString::OTC_RString(char const* theString, u_int theLength)
{
  OTCLIB_ENSURE_FN((theString != 0 || theLength == 0),
   "OTC_RString::OTC_RString(char const*, u_int)",
   "Invalid string.");

  if (theLength == 0)
    theString = "";

  mutex_.lock();

  OTC_MutexReaper<OTC_NRMutex> xxxMutex1;
  xxxMutex1.grab(mutex_);

  if (index_ == 0)
  {
    index_ = new OTC_HashTable;
    OTCLIB_ASSERT_M(index_ != 0);
  }

  xxxMutex1.release();

  mutex_.unlock();

  sdmutex_.lock();

  OTC_MutexReaper<OTC_NRMutex> xxxMutex2;
  xxxMutex2.grab(sdmutex_);

  int tmpRank;
  int tmpHashValue;
  u_int tmpIteration = 0;
  u_int tmpStartIndex = 0;
  u_int tmpEntryIndex = 0;

  int tmpAvailableIndex = -1;

  OTC_HashEntry* tmpEntry = 0;
  OTC_StringData* tmpIndexEntry = 0;

  tmpHashValue = OTC_Hash::hash(theString,theLength);
  tmpEntry = index_->searchHash(tmpHashValue,
   tmpIteration,tmpStartIndex,tmpEntryIndex);

  while (tmpEntry != 0)
  {
    if (!tmpEntry->isOccupied())
    {
      if (tmpAvailableIndex == -1)
	tmpAvailableIndex = tmpEntryIndex;
    }

    if (tmpEntry->isEmpty())
      break;

    if (tmpEntry->isOccupied())
    {
      tmpIndexEntry = (OTC_StringData*)tmpEntry->item();

      tmpRank = OTC_Rank::rank(theString,theLength,
       tmpIndexEntry->data,tmpIndexEntry->length);

      if (tmpRank != 0)
	tmpIndexEntry = 0;
      else
	break;
    }

    tmpEntry = index_->searchHash(tmpHashValue,
     tmpIteration,tmpStartIndex,tmpEntryIndex);
  }

  if (tmpIndexEntry == 0)
  {
    tmpIndexEntry = new OTC_StringData;
    OTCLIB_ASSERT_M(tmpIndexEntry != 0);

    tmpIndexEntry->count = 1;
    tmpIndexEntry->length = theLength;
    tmpIndexEntry->capacity = theLength+1;
    tmpIndexEntry->symbol = OTCLIB_TRUE;

    tmpIndexEntry->data = allocate_(tmpIndexEntry->capacity);
    OTCLIB_ASSERT_M(tmpIndexEntry->data != 0);

    OTC_Copy::copy(tmpIndexEntry->data,theString,theLength);
    tmpIndexEntry->data[theLength] = EOS;

    index_->add(tmpAvailableIndex,tmpIndexEntry,tmpHashValue);
  }

  // Note that initial reference count will be 2 and not 1. This
  // is so that delayed copy will always occur if something tries
  // to modify the string.

  data_ = tmpIndexEntry;
  data_->count++;

  xxxMutex2.release();

  sdmutex_.unlock();
}

/* ------------------------------------------------------------------------- */
void OTC_RString::reallocate(u_int theLength, OTC_StringBuffering theType)
{
#if defined(ENV_OSTORE)
  OTC_Locality theLocality;
  OTC_Boolean theLocalityIsSet = OTCLIB_FALSE;
#endif

  // If memory is shared, always create new memory area. Only need to
  // hold lock while operating on shared data.

  OTC_Boolean theShared;

  sdmutex_.lock();

  OTC_MutexReaper<OTC_NRMutex> xxxMutex;
  xxxMutex.grab(sdmutex_);

  // Remember the previous string capacity in case we are going to
  // release it.

  u_int tmpCapacity = data_->capacity;

  theShared = (data_->count != 1);

  if (theShared != OTCLIB_FALSE)
  {
    OTC_StringData* tmpData;
#if defined(ENV_OSTORE)
    if (theLocalityIsSet == OTCLIB_FALSE)
    {
      theLocality = OTC_Locality::of(this);
      theLocalityIsSet = OTCLIB_TRUE;
    }
    tmpData = new (theLocality,OTC_StringData::get_os_typespec())
     OTC_StringData;
#else
    tmpData = new OTC_StringData;
#endif
    OTCLIB_ASSERT_M(tmpData != 0);

    tmpData->count = 1;
    tmpData->length = theLength;
    tmpData->symbol = OTCLIB_FALSE;

    if (theType == OTCLIB_BUFFERED)
    {
      tmpData->capacity = capacity_(theLength)+1;
      OTCLIB_ASSERT(tmpData->capacity >= theLength+1);
    }
    else
    {
      tmpData->capacity = theLength+1;
    }

#if defined(ENV_OSTORE)
    tmpData->data = new (theLocality,os_typespec::get_char(),
     tmpData->capacity) char[tmpData->capacity];
#else
    tmpData->data = allocate_(tmpData->capacity);
#endif
    OTCLIB_ASSERT_M(tmpData->data != 0);

    tmpData->data[tmpData->length] = EOS;

    if (tmpData->length >= data_->length)
      OTC_Copy::copy(tmpData->data,data_->data,data_->length);
    else
      OTC_Copy::copy(tmpData->data,data_->data,tmpData->length);

    data_->count--;
    data_ = tmpData;
  }

  xxxMutex.release();

  sdmutex_.unlock();

  if (theShared != OTCLIB_FALSE)
    return;

  // If not shared and length has not changed, don't bother doing
  // anything.

  if (theLength == data_->length)
    return;

  // If length is reduced, add in a new null terminator and adjust
  // length.

  if (theLength <= data_->length)
  {
    data_->data[theLength] = EOS;
    data_->length = theLength;

    return;
  }

  // If length has increased, but have sufficient capacity, add in a new null
  // terminator and adjust length.

  if (theLength > data_->length && theLength < data_->capacity)
  {
    data_->data[theLength] = EOS;
    data_->length = theLength;

    return;
  }

  // If we get here, it means we have to allocate some new memory.

  if (theType == OTCLIB_BUFFERED)
  {
    data_->capacity = capacity_(theLength)+1;
    OTCLIB_ASSERT(data_->capacity >= theLength+1);
  }
  else
  {
    data_->capacity = theLength+1;
  }

  char* tmpString;
#if defined(ENV_OSTORE)
  if (theLocalityIsSet == OTCLIB_FALSE)
  {
    theLocality = OTC_Locality::of(this);
    theLocalityIsSet = OTCLIB_TRUE;
  }
  tmpString = new (theLocality,os_typespec::get_char(),
   data_->capacity) char[data_->capacity];
#else
  tmpString = allocate_(data_->capacity);
#endif
  OTCLIB_ASSERT_M(tmpString != 0);
  tmpString[theLength] = EOS;

  OTC_Copy::copy(tmpString,data_->data,data_->length);

  release_(data_->data,tmpCapacity);
  data_->data = tmpString;

  data_->length = theLength;
}

/* ------------------------------------------------------------------------- */
OTC_RString const& OTC_RString::operator=(OTC_RString const& theString)
{
  if (&theString == this)
    return *this;

  OTC_MutexReaper<OTC_NRMutex> xxxMutex;

#if defined(ENV_OSTORE)
  // Make sure segment check is against <theString.data_> as constructor
  // accepting <void*> for locality means that data could be in different
  // class than what is holding it. If localities are different, we will
  // always have to make a copy of the string. Reuse the string data object
  // for this class if we can.

  OTC_Locality theLocality = OTC_Locality::of(this);
  OTC_Locality theRHSLocality = OTC_Locality::of(theString.data_);

  if (theLocality != theRHSLocality)
  {
    char* oldString = 0;
    u_int oldCapacity = 0;

    sdmutex_.lock();

    xxxMutex.grab(sdmutex_);

    if (data_ != 0)
    {
      if (data_->count == 2 && data_->symbol != OTCLIB_FALSE)
      {
	int tmpHashValue;
	tmpHashValue = OTC_Hash::hash(data_->data,data_->length);
	index_->remove(data_,tmpHashValue);

        oldString = data_->data;
	oldCapacity = data_->capacity;
      }
      else if (data_->count == 1)
      {
        oldString = data_->data;
	oldCapacity = data_->capacity;
      }
      else
      {
	data_->count--;

	data_ = new (theLocality,OTC_StringData::get_os_typespec())
	 OTC_StringData;
	OTCLIB_ASSERT_M(data_ != 0);
      }
    }
    else
    {
      data_ = new (theLocality,OTC_StringData::get_os_typespec())
       OTC_StringData;
      OTCLIB_ASSERT_M(data_ != 0);
    }

    // Fix up string data object and copy source string across.

    data_->count = 1;
    data_->length = theString.data_->length;
    data_->symbol = OTCLIB_FALSE;

    data_->capacity = capacity_(data_->length)+1;
    OTCLIB_ASSERT(data_->capacity >= data_->length+1);

    data_->data = new (theLocality,os_typespec::get_char(),
     data_->capacity) char[data_->capacity];
    OTCLIB_ASSERT_M(data_->data != 0);

    OTC_Copy::copy(data_->data,theString.data_->data,data_->length);

    data_->data[data_->length] = EOS;

    xxxMutex.release();

    sdmutex_.unlock();

    if (oldString != 0)
      release_(oldString,oldCapacity);

    return *this;
  }
#endif

  OTC_StringData* oldData = 0;

  sdmutex_.lock();

  xxxMutex.grab(sdmutex_);

  if (data_ != 0)
  {
    if (data_->count == 2 && data_->symbol != OTCLIB_FALSE)
    {
      int tmpHashValue;
      tmpHashValue = OTC_Hash::hash(data_->data,data_->length);
      index_->remove(data_,tmpHashValue);

      oldData = data_;
    }
    else if (data_->count == 1)
      oldData = data_;
    else
      data_->count--;
  }

  theString.data_->count++;
  data_ = theString.data_;

  xxxMutex.release();

  sdmutex_.unlock();

  if (oldData != 0)
  {
    release_(oldData->data,oldData->capacity);
    delete oldData;
  }

  return *this;
}

/* ------------------------------------------------------------------------- */
u_int OTC_RString::capacity_(u_int theCapacity) const
{
  mutex_.lock();

  if (initialised_ == OTCLIB_FALSE)
  {
    initialised_ = OTCLIB_TRUE;
    bufType_ = OTCLIB_BUFFERING1;

    char const* env = 0;

    env = getenv("OTCLIB_NOSTRINGBUFFERING");
    if (env != 0)
    {
      bufType_ = OTCLIB_BUFFERING0;
    }
    else
    {
      env = getenv("OTCLIB_STRINGBUFFERING1");
      if (env != 0)
	bufType_ = OTCLIB_BUFFERING2;
    }
  }

  mutex_.unlock();

  switch (bufType_)
  {
    case OTCLIB_BUFFERING1:
    {
      // These values are magic. For those less
      // than 64 bytes, we only take off 1 byte to
      // allow for EOS marker. For those above 64
      // bytes, we take off 1 byte for EOS marker,
      // plus 8 bytes in case memory allocator uses
      // power of 2 blocks, with 8 bytes for its
      // internal memory chain. Above 4096 bytes,
      // just allow for some slop.

      if (theCapacity > 15)
      {
	if (theCapacity > 63)
	{
	  if (theCapacity > 247)
	  {
	    if (theCapacity > 1015)
	    {
	      if (theCapacity > 4087)
		theCapacity += (theCapacity >> 2);
	      else
		theCapacity += (theCapacity >> 1);
	    }
	    else
	      theCapacity = 1024;
	  }
	  else
	    theCapacity = 256;
	}
	else
	  theCapacity = 63;
      }
      else
	theCapacity = 15;

      break;
    }

    case OTCLIB_BUFFERING2:
    {
      if ((theCapacity % 16) != 0)
	theCapacity = (theCapacity/16 + 1) * 16;

      break;
    }

    default:
    {
      break;
    }
  }

  return theCapacity;
}

/* ------------------------------------------------------------------------- */
OTC_Boolean OTC_RString::symbolExists(char const* theString, u_int theLength)
{
  OTCLIB_ENSURE_FN((theString != 0 || theLength == 0),
   "OTC_RString::symbolExists(char const*, u_int)",
   "null pointer but non zero count");

  mutex_.lock();

  OTC_MutexReaper<OTC_NRMutex> xxxMutex1;
  xxxMutex1.grab(mutex_);

  if (index_ == 0)
  {
    index_ = new OTC_HashTable;
    OTCLIB_ASSERT_M(index_ != 0);
  }

  xxxMutex1.release();

  mutex_.unlock();

  OTC_Boolean theResult;
  theResult = OTCLIB_FALSE;

  sdmutex_.lock();

  OTC_MutexReaper<OTC_NRMutex> xxxMutex2;
  xxxMutex2.grab(sdmutex_);

  if (index_->population() != 0)
  {
    int tmpRank;
    int tmpHashValue;
    u_int tmpIteration = 0;
    u_int tmpStartIndex = 0;
    u_int tmpEntryIndex = 0;

    OTC_HashEntry* tmpEntry = 0;
    OTC_StringData* tmpIndexEntry = 0;

    tmpHashValue = OTC_Hash::hash(theString,theLength);
    tmpEntry = index_->searchHash(tmpHashValue,
     tmpIteration,tmpStartIndex,tmpEntryIndex);

    while (tmpEntry != 0 && !tmpEntry->isEmpty())
    {
      if (tmpEntry->isOccupied())
      {
	tmpIndexEntry = (OTC_StringData*)tmpEntry->item();
	tmpRank = OTC_Rank::rank(theString,theLength,
	 tmpIndexEntry->data,tmpIndexEntry->length);
	if (tmpRank == 0)
	{
	  theResult = OTCLIB_TRUE;
	  break;
	}
      }

      tmpEntry = index_->searchHash(tmpHashValue,
       tmpIteration,tmpStartIndex,tmpEntryIndex);
    }
  }

  xxxMutex2.release();

  sdmutex_.unlock();

  return theResult;
}

/* ------------------------------------------------------------------------- */
OSE_EXPORT ostream& operator<<(ostream& outs, OTC_RString const& theString)
{
  if (!outs.good())
    return outs;

  // Special streams prefix operations.

#if 0
  if (!outs.opfx())
    return outs;
#endif

  // If number of characters in string greater than width defined in
  // stream then just write it out. If not we must do some formatting.

  u_int theWidth = outs.width();
  u_int theLength = theString.length();

  if (theLength >= theWidth)
  {
    outs.write(theString.string(),theLength);
  }
  else
  {
    if (!(outs.flags() & ios::left))
    {
      for (int i=0; i<int(theWidth-theLength); i++)
	outs.put((char)outs.fill());
    }
    outs.write(theString.string(),theLength);
    if (outs.flags() & ios::left)
    {
      for (int i=0; i<int(theWidth-theLength); i++)
	outs.put((char)outs.fill());
    }
  }

  // Reset the stream width setting.

  outs.width(0);

  // Sepcial streams suffix operations.

#if 0
  outs.osfx();
#endif

  return outs;
}

/* ------------------------------------------------------------------------- */
char* OTC_RString::allocate_(u_int theSize) const
{
  char* theMemory = 0;

  if (bufType_ == OTCLIB_BUFFERING1)
    theMemory = (char*)OTC_CommonPool::allocate(theSize);
  else
    theMemory = new char[theSize];

  return theMemory;
}

/* ------------------------------------------------------------------------- */
void OTC_RString::release_(char* theMemory, u_int theSize) const
{
#if defined(ENV_OSTORE)
  delete [] theMemory;
#else
  if (bufType_ == OTCLIB_BUFFERING1)
    OTC_CommonPool::release(theMemory,theSize);
  else
    delete [] theMemory;
#endif
}

/* ------------------------------------------------------------------------- */
