# COPYIGHT 2001 DUMPLETON SOFTWARE CONSULTING PTY LIMITED

import os
import sys
import types
import string
import posixpath
import md5
import base64
import imp
import time

if os.environ.has_key("OSE_ROOT"):
  OSE_ROOT = os.environ["OSE_ROOT"]
else:
  OSE_ROOT = "@OSE_ROOT@"

OSE_RELEASE_DIRECTORY = "@OSE_RELEASE_DIRECTORY@"

if os.environ.has_key("OSE_HOST"):
  OSE_HOST = os.environ["OSE_HOST"]
else:
  OSE_HOST = "UNKNOWN"

OSE_HOME = OSE_ROOT + "/" + OSE_RELEASE_DIRECTORY
OSE_HOST_HOME = OSE_HOME + "/" + OSE_HOST

sys.path.insert(0,OSE_HOST_HOME+"/lib/python")

# Import "socket" so that WinSock
# gets initialised on Win32 platform.

import socket

import _ose
from netrpc.common import *


# Setup constants for dispatcher.

IDLE_JOB = 0
STANDARD_JOB = 1
PRIORITY_JOB = 2

# Force initialisation of dispatcher.

_dispatcher = _ose.Dispatcher()

def Dispatcher():
  return _dispatcher

class Job:

  def __init__(self,callback,params=()):
    self._callback = callback
    self._params = params
    self._runnable = 1

  def execute(self,params=None):
    if self._runnable:
      if params == None:
        params = self._params
      apply(self._callback,params)

  def schedule(self,type,params=None):
    if params != None:
      self._params = params
    if self._runnable:
      _dispatcher.schedule(self,type)

  def cancel(self):
    self._runnable = 0

  def reset(self):
    self._runnable = 1

  def __nonzero__(self):
    return self._runnable != 0

  def __call__(self):
    self.execute(self._params)


# Define way of trying to stop the whole application.

def terminate():
  _dispatcher.stop()
  raise SystemExit

# Define wrapper for dispatcher job callback function.

def _apply(callback,arguments):

  try:
    return apply(callback,arguments)
  except SystemExit:
    terminate()
  except:
    logException(1)
    terminate()

_ose.init_JobCallback(_apply)


# Force initialisation of program data.

_program = _ose.Program()

ID_LONG_FORMAT = 0
ID_SHORT_FORMAT = 1

uniqueId = _program.uniqueId
processIdentity = _program.processIdentity

removeAllConfig = _program.removeAllConfig
removeConfig = _program.removeConfig
loadConfig = _program.loadConfig
mergeConfig = _program.mergeConfig
lookupConfig = _program.lookupConfig
saveConfig = _program.saveConfig

lookupEnviron = _program.lookupEnviron
mergeEnviron = _program.mergeEnviron
expandEnviron = _program.expandEnviron

# Setup constants for logger.

LOG_EMERGENCY = 0
LOG_ALERT = 1
LOG_CRITICAL = 2
LOG_ERROR = 3
LOG_WARNING = 4
LOG_NOTICE = 5
LOG_INFO = 6
LOG_DEBUG = 7

# Define logger, setup framework for channel monitors and function
# for dumping details of an exception out to the C++ logger.

_logSubscribers = {}

def _handleLogMessage(channel,level,message):
  for logger in _logSubscribers.keys():
    logger._handleLogMessage(channel,level,message)

_logger = _ose.new_Logger(_handleLogMessage)
if not os.environ.has_key("OTCLIB_NOLOGSTDERR"):
  _logger.enableStderrOutput()


def logException(skip=0):
  error = exceptionDetails(skip)
  _logger.notify(LOG_ERROR,error)


class Logger:

  def __init__(self):
    self._callbacks = {}

    self.notify = _logger.notify
    self.notifyChannel = _logger.notifyChannel
    self.logFile = _logger.logFile
    self.setLogFile = _logger.setLogFile
    self.stderrOutputEnabled = _logger.stderrOutputEnabled
    self.enableStderrOutput = _logger.enableStderrOutput
    self.disableStderrOutput = _logger.disableStderrOutput
    self.longFormatEnabled = _logger.longFormatEnabled
    self.enableLongFormat = _logger.enableLongFormat
    self.disableLongFormat = _logger.disableLongFormat

  def destroyReferences(self):
    global _logSubscribers
    self._callbacks = {}
    if _logSubscribers.has_key(self):
       del _logSubscribers[self]

  def monitorChannel(self,channel,callback):
    global _logSubscribers
    if callback == None:
      if self._callbacks.has_key(channel):
        del self._callbacks[channel]
        if len(self._callbacks.keys()) == 0:
          del _logSubscribers[self]
    else:
      self._callbacks[channel] = callback
      _logSubscribers[self] = 1

  def _handleLogMessage(self,channel,level,message):
    try:
      if self._callbacks.has_key(channel):
        callback = self._callbacks[channel]
        callback(channel,level,message)
    except SystemExit:
      terminate()
    except:
      logException()
      terminate()


# Setup constants for agents.

SOCKET_POLLIN = 1
SOCKET_POLLPRI = 2
SOCKET_POLLOUT = 4
SOCKET_POLLERR = 8
SOCKET_POLLHUP = 16
SOCKET_POLLNVAL = 32

SHUTDOWN_PENDING = 0
SHUTDOWN_ARRIVED = 1

# Agent wrapper.

class Agent:

  def __init__(self,inner=None):
    if inner == None:
      inner = _ose.new_Agent(self)
    self._Agent = inner
    self._timerCallbacks = {}
    self._alarmCallbacks = {}
    self._cronCallbacks = {}
    self._actionCallbacks = {}
    self._socketInCallbacks = {}
    self._socketPriCallbacks = {}
    self._socketOutCallbacks = {}
    self._signalCallbacks = {}
    self._shutdownCallback = None

  def __del__(self):
    self._timerCallbacks = {}
    self._alarmCallbacks = {}
    self._cronCallbacks = {}
    self._actionCallbacks = {}
    self._socketInCallbacks = {}
    self._socketPriCallbacks = {}
    self._socketOutCallbacks = {}
    self._signalCallbacks = {}
    if self._Agent != None:
      self._Agent.cancelAllTimers()
      self._Agent.cancelAllAlarms()
      self._Agent.cancelAllActions()
      self._Agent.unsubscribeAllSockets()
      self._Agent.unsubscribeAllSignals()
      self._Agent.unsubscribeShutdown()

  def destroyReferences(self):
    self.cancelAllTimers()
    self.cancelAllAlarms()
    self.cancelAllActions()
    self.unsubscribeAllSockets()
    self.unsubscribeAllSignals()
    self.unsubscribeShutdown()

  def startTimer(self,callback,delay,name):
    self.cancelTimer(name)
    self._timerCallbacks[name] = callback
    self._Agent.startTimer(delay,name)

  def cancelTimer(self,name):
    if self._timerCallbacks.has_key(name):
      del self._timerCallbacks[name]
      self._Agent.cancelTimer(name)

  def cancelAllTimers(self):
    self._timerCallbacks = {}
    self._Agent.cancelAllTimers()

  def _handleTimer(self,name):
    try:
      if self._timerCallbacks.has_key(name):
	callback = self._timerCallbacks[name]
	del self._timerCallbacks[name]
	callback(name)
    except SystemExit:
      terminate()
    except:
      logException()
      terminate()

  def setAlarm(self,callback,time,name):
    self.cancelAlarm(name)
    self._alarmCallbacks[name] = callback
    self._Agent.setAlarm(time,name)

  def cancelAlarm(self,name):
    if self._alarmCallbacks.has_key(name):
      del self._alarmCallbacks[name]
      self._Agent.cancelAlarm(name)

  def cancelAllAlarms(self):
    self._alarmCallbacks = {}
    self._Agent.cancelAllAlarms()

  def _handleAlarm(self,name):
    try:
      if self._alarmCallbacks.has_key(name):
	callback = self._alarmCallbacks[name]
	del self._alarmCallbacks[name]
	callback(name)
    except SystemExit:
      terminate()
    except:
      logException()
      terminate()

  def scheduleAction(self,callback,mode,name):
    self.cancelAction(name)
    if type(mode) is types.IntType:
      self._actionCallbacks[name] = callback
      self._Agent.scheduleAction(mode,name)
    else:
      self._cronCallbacks[name] = callback
      self._Agent.scheduleCronAction(mode,name)

  def cancelAction(self,name):
    if self._cronCallbacks.has_key(name):
      del self._cronCallbacks[name]
      self._Agent.cancelAction(name)
    elif self._actionCallbacks.has_key(name):
      del self._actionCallbacks[name]
      self._Agent.cancelAction(name)

  def cancelAllActions(self):
    self._cronCallbacks = {}
    self._actionCallbacks = {}
    self._Agent.cancelAllActions()

  def _handleAction(self,name):
    try:
      if self._cronCallbacks.has_key(name):
	callback = self._cronCallbacks[name]
	callback(name)
      elif self._actionCallbacks.has_key(name):
	callback = self._actionCallbacks[name]
	del self._actionCallbacks[name]
	callback(name)
    except SystemExit:
      terminate()
    except:
      logException()
      terminate()

  def subscribeSocket(self,callback,fileno,event=SOCKET_POLLIN):
    self.unsubscribeSocket(fileno,event)
    if event & SOCKET_POLLIN:
      self._socketInCallbacks[fileno] = callback
    if event & SOCKET_POLLPRI:
      self._socketPriCallbacks[fileno] = callback
    if event & SOCKET_POLLOUT:
      self._socketOutCallbacks[fileno] = callback
    self._Agent.subscribeSocket(fileno,event)

  def unsubscribeSocket(self,fileno,event=~0):
    if event & SOCKET_POLLIN:
      if self._socketInCallbacks.has_key(fileno):
	del self._socketInCallbacks[fileno]
    if event & SOCKET_POLLPRI:
      if self._socketPriCallbacks.has_key(fileno):
	del self._socketPriCallbacks[fileno]
    if event & SOCKET_POLLOUT:
      if self._socketOutCallbacks.has_key(fileno):
	del self._socketOutCallbacks[fileno]
    self._Agent.unsubscribeSocket(fileno,event)

  def unsubscribeAllSockets(self):
    self._socketInCallbacks = {}
    self._socketPriCallbacks = {}
    self._socketOutCallbacks = {}
    self._Agent.unsubscribeAllSockets()

  def _handleSocket(self,fileno,event):
    try:
      if event & SOCKET_POLLOUT:
	if self._socketOutCallbacks.has_key(fileno):
	  callback = self._socketOutCallbacks[fileno]
	  callback(fileno,SOCKET_POLLOUT)
      event = event & ~SOCKET_POLLOUT
      if event & SOCKET_POLLPRI:
	if self._socketPriCallbacks.has_key(fileno):
	  callback = self._socketPriCallbacks[fileno]
	  callback(fileno,SOCKET_POLLPRI)
      event = event & ~SOCKET_POLLPRI
      if event != 0:
	if self._socketInCallbacks.has_key(fileno):
	  callback = self._socketInCallbacks[fileno]
	  callback(fileno,event)
    except SystemExit:
      terminate()
    except:
      logException()
      terminate()

  def subscribeSignal(self,callback,num):
    self.unsubscribeSignal(num)
    self._signalCallbacks[num] = callback
    self._Agent.subscribeSignal(num)

  def unsubscribeSignal(self,num):
    if self._signalCallbacks.has_key(num):
      del self._signalCallbacks[num]
      self._Agent.unsubscribeSignal(num)

  def unsubscribeAllSignals(self):
    self._signalCallbacks = {}
    self._Agent.unsubscribeAllSignals()

  def _handleSignal(self,num):
    try:
      if self._signalCallbacks.has_key(num):
	callback = self._signalCallbacks[num]
	callback(num)
    except SystemExit:
      terminate()
    except:
      logException()
      terminate()

  def scheduleShutdown(self,delay=0):
    self._Agent.scheduleShutdown(delay)

  def suspendShutdown(self):
    self._Agent.suspendShutdown()

  def resumeShutdown(self):
    self._Agent.resumeShutdown()

  def subscribeShutdown(self,callback):
    self.unsubscribeShutdown()
    self._shutdownCallback = callback
    self._Agent.subscribeShutdown()

  def unsubscribeShutdown(self):
    if self._shutdownCallback != None:
      self._shutdownCallback = None
      self._Agent.unsubscribeShutdown()

  def _handleShutdown(self,category):
    try:
      if self._shutdownCallback != None:
	callback = self._shutdownCallback
	callback(category)
    except SystemExit:
      terminate()
    except:
      logException()
      terminate()


# Setup constants for services.

SERVICE_LOCAL = 0
SERVICE_REMOTE = 1

SERVICE_AVAILABLE = 0
SERVICE_WITHDRAWN = 1

SUBSCRIPTION_REQUESTED = 0
SUBSCRIPTION_WITHDRAWN = 1


# Define classes for registry monitor.

class Monitor(Agent):

  def __init__(self,inner=None):
    if inner == None:
      inner = _ose.new_Monitor(self)
    Agent.__init__(self,inner)
    self._serviceNameCallbacks = {}
    self._serviceAddressCallbacks = {}
    self._serviceGroupCallbacks = {}
    self._registryCallback = None

  def __del__(self):
    self._registryCallback = None
    self._serviceNameCallbacks = {}
    self._serviceAddressCallbacks = {}
    self._serviceGroupCallbacks = {}
    if self._Agent != None:
      self._Agent.unsubscribeAllRegistry()

  def destroyReferences(self):
    Agent.destroyReferences(self)
    self._serviceNameCallbacks = {}
    self._serviceAddressCallbacks = {}
    self._serviceGroupCallbacks = {}
    self._Agent.unsubscribeAllRegistry()

  def serviceAgents(self):
    return self._Agent.serviceAgents()

  def serviceGroups(self,serviceAddress=None):
    return self._Agent.serviceGroups(serviceAddress)

  def lookupServiceName(self,serviceName):
    return self._Agent.lookupServiceName(serviceName)

  def lookupServiceAddress(self,serviceAddress):
    return self._Agent.lookupServiceAddress(serviceAddress)

  def lookupServiceGroup(self,serviceGroup):
    return self._Agent.lookupServiceGroup(serviceGroup)

  def subscribeRegistry(self,callback):
    self.unsubscribeRegistry()
    self._registryCallback = callback
    self._Agent.subscribeRegistry()

  def unsubscribeRegistry(self):
    if self._registryCallback != None:
      self._registryCallback = None
      self._Agent.unsubscribeRegistry()

  def _handleRegistry(self,binding,group,status):
    try:
      if self._registryCallback != None:
	callback = self._registryCallback
	callback(binding,group,status)
    except SystemExit:
      terminate()
    except:
      logException()
      terminate()

  def subscribeServiceName(self,callback,serviceName):
    self.unsubscribeServiceName(serviceName)
    self._serviceNameCallbacks[serviceName] = callback
    self._Agent.subscribeServiceName(serviceName)

  def unsubscribeServiceName(self,serviceName):
    if self._serviceNameCallbacks.has_key(serviceName):
      del self._serviceNameCallbacks[serviceName]
      self._Agent.unsubscribeServiceName(serviceName)

  def _handleServiceName(self,binding,status):
    try:
      serviceName = binding.serviceName()
      if self._serviceNameCallbacks.has_key(serviceName):
	callback = self._serviceNameCallbacks[serviceName]
	callback(binding,status)
    except SystemExit:
      terminate()
    except:
      logException()
      terminate()

  def subscribeServiceAddress(self,callback,serviceAddress):
    self.unsubscribeServiceAddress(serviceAddress)
    self._serviceAddressCallbacks[serviceAddress] = callback
    self._Agent.subscribeServiceAddress(serviceAddress)

  def unsubscribeServiceAddress(self,serviceAddress):
    if self._serviceAddressCallbacks.has_key(serviceAddress):
      del self._serviceAddressCallbacks[serviceAddress]
      self._Agent.unsubscribeServiceAddress(serviceAddress)

  def _handleServiceAddress(self,binding,group,status):
    try:
      serviceAddress = binding.serviceAddress()
      if self._serviceAddressCallbacks.has_key(serviceAddress):
	callback = self._serviceAddressCallbacks[serviceAddress]
	callback(binding,group,status)
    except SystemExit:
      terminate()
    except:
      logException()
      terminate()

  def subscribeServiceGroup(self,callback,serviceGroup):
    self.unsubscribeServiceGroup(serviceGroup)
    self._serviceGroupCallbacks[serviceGroup] = callback
    self._Agent.subscribeServiceGroup(serviceGroup)

  def unsubscribeServiceGroup(self,serviceGroup):
    if self._serviceGroupCallbacks.has_key(serviceGroup):
      del self._serviceGroupCallbacks[serviceGroup]
      self._Agent.unsubscribeServiceGroup(serviceGroup)

  def _handleServiceGroup(self,binding,group,status):
    try:
      if self._serviceGroupCallbacks.has_key(group):
	callback = self._serviceGroupCallbacks[group]
	callback(binding,group,status)
    except SystemExit:
      terminate()
    except:
      logException()
      terminate()


class _Registry:

  def __init__(self):
    self._Monitor = _ose.new_Monitor(self)

  def lookupServiceName(self,serviceName):
    return self._Monitor.lookupServiceName(serviceName)

  def lookupServiceAddress(self,serviceAddress):
    return self._Monitor.lookupServiceAddress(serviceAddress)

  def lookupServiceGroup(self,serviceGroup):
    return self._Monitor.lookupServiceGroup(serviceGroup)


_registry = _Registry()

def Registry():
  return _registry


# Define outer classes for corresponding inner C++ classes.

class _Request:

  def __init__(self,inner,service):
    self._inner = inner
    self._service = service
    self.sender = self._inner.sender
    self.conversationId = self._inner.conversationId
    self.sequenceNumber = self._inner.sequenceNumber
    self.methodName = self._inner.methodName
    self.bounceRequest = self._inner.bounceRequest
    self.sendFailure = self._inner.sendFailure

  def content(self):
    return self._inner.content(self._service.decodeValue)

  def sendResponse(self,content):
    self._inner.sendResponse(content,self._service.encodeObject)


class _Response:

  def __init__(self,inner,service):
    self._inner = inner
    self._service = service
    self.sender = self._inner.sender
    self.conversationId = self._inner.conversationId
    self.sequenceNumber = self._inner.sequenceNumber

  def content(self):
    return self._inner.content(self._service.decodeValue)


class _Report:

  def __init__(self,inner,service):
    self._inner = inner
    self._service = service
    self.publisher = self._inner.publisher
    self.subjectName = self._inner.subjectName
    self.subjectPattern = self._inner.subjectPattern

  def content(self):
    return self._inner.content(self._service.decodeValue)


class _RemoteFailure:

  def __init__(self,inner):
    self._inner = inner
    self.sender = self._inner.sender
    self.conversationId = self._inner.conversationId
    self.sequenceNumber = self._inner.sequenceNumber
    self.error = self._inner.error
    self.description = self._inner.description
    self.origin = self._inner.origin
    self.details = self._inner.details

  def __repr__(self):
    return ("<_RemoteFailure %s, description=%s, origin=%s, details=%s>" %
        (self.error(),`self.description()`,`self.origin()`,`self.details()`))

  __str__ = __repr__


class _LocalFailure:

  def __init__(self,response,error,description,origin,details):
    self._response = response
    self._error = error
    self._description = description
    self._origin = origin
    self._details = details
    self.sender = self._response.sender
    self.conversationId = self._response.conversationId
    self.sequenceNumber = self._response.sequenceNumber

  def error(self):
    return self._error

  def description(self):
    return self._description

  def origin(self):
    return self._origin

  def details(self):
    return self._details

  def __repr__(self):
    return ("<_LocalFailure %s, description=%s, origin=%s, details=%s>" %
        (self.error(),`self.description()`,`self.origin()`,`self.details()`))

  __str__ = __repr__


class _RequestTimeout:

  def __init__(self,binding,conversationId,
      sequenceNumber,error,description,origin="",details=""):
    self._binding = binding
    self._conversationId = conversationId
    self._sequenceNumber = sequenceNumber
    self._error = error
    self._description = description
    self._origin = origin
    self._details = details
    self.sender = self._binding.serviceAddress

  def conversationId(self):
    return self._conversationId

  def sequenceNumber(self):
    return self._sequenceNumber

  def error(self):
    return self._error

  def description(self):
    return self._description

  def origin(self):
    return self._origin

  def details(self):
    return self._details

  def __repr__(self):
    return ("<_RequestTimeout %s, description=%s, origin=%s, details=%s>" %
        (self.error(),`self.description()`,`self.origin()`,`self.details()`))

  __str__ = __repr__


class DelayedResponse:
  pass


class _ServiceMethod:

  def __init__(self,local,serviceName,binding,methodName):
    self._local = local
    self._serviceName = serviceName
    self._binding = binding
    self._methodName = methodName

  def __getattr__(self,methodName):
    return _ServiceMethod(self._local,self._serviceName,self._binding,
        "%s.%s"%(self._methodName,methodName))

  def __call__(self,*args):
    if self._binding == None:
      self._local._lastRequest = None
      error = CLIENT_SERVER_UNAVAILABLE
      description = CLIENT_SERVER_UNAVAILABLE_MESSAGE
      origin = "netsvc"
      details = self._serviceName
      raise ServiceUnavailable(error,description,origin,details)
    id = self._local.sendRequest(self._binding,self._methodName,args)
    self._local._lastRequest = (id,self._binding,self._methodName)
    return id


class _ServiceEndPoint:

  def __init__(self,local,serviceName,binding):
    self._local = local
    self._serviceName = serviceName
    self._binding = binding

  def __getattr__(self,methodName):
    return _ServiceMethod(self._local,
        self._serviceName,self._binding,methodName)

  def __coerce__(self,other):
    if other == None:
      return (self._binding,other)
    return None

  def __nonzero__(self):
    return self._binding != None


class Service(Monitor):

  _services = {}

  def __init__(self,name="",audience="*"):
    inner = _ose.new_Service(self,name,audience)
    Monitor.__init__(self,inner)
    self._name = name
    self._identity = self.serviceBinding().agentIdentity()
    if not Service._services.has_key(name):
      Service._services[name] = [self]
    else:
      Service._services[name].append(self)
    self._exportedMethods = {}
    self._lastRequest = None
    self._activeRequests = {}
    self._activeSubscriptions = {}
    self._passiveSubscriptions = {}
    self._currentRequest = None
    self._currentResponse = None
    self._currentFailure = None
    self._currentReport = None
    self._currentTimeout = None

  def __del__(self):
    self._exportedMethods = {}
    self._lastRequest = None
    self._activeRequests = {}
    self._activeSubscriptions = {}
    self._passiveSubscriptions = {}
    self._currentRequest = None
    self._currentResponse = None
    self._currentFailure = None
    self._currentReport = None
    self._currentTimeout = None

  def destroyReferences(self):
    Monitor.destroyReferences(self)
    index = Service._services[self._name].index(self)
    del Service._services[self._name][index]
    if len(Service._services[self._name]) == 0:
      del Service._services[self._name]
    self._exportedMethods = {}
    self._lastRequest = None
    self._activeRequests = {}
    self._activeSubscriptions = {}
    self._passiveSubscriptions = {}
    self._currentRequest = None
    self._currentResponse = None
    self._currentFailure = None
    self._currentReport = None
    self._currentTimeout = None

  def serviceBinding(self):
    return self._Agent.serviceBinding()

  def serviceName(self):
    return self._name

  def agentIdentity(self):
    return self._identity

  def exportMethod(self,method,methodName=None):
    if methodName == None:
      methodName = method.__name__
    self._exportedMethods[methodName] = method

  def exportedMethods(self):
    return self._exportedMethods.keys()

  def lookupMethod(self,methodName):
    if self._exportedMethods.has_key(methodName):
      return self._exportedMethods[methodName]
    return None

  def serviceEndPoint(self,service):
    binding = None
    if type(service) != type(self.serviceBinding()):
      bindings = self.lookupServiceName(service)
      if len(bindings) != 0:
        binding = bindings[0]
    else:
      binding = service
      service = binding.serviceName()
    return _ServiceEndPoint(self,service,binding)

  def decodeValue(self,name,string):
    return decodeValue(name,string)

  def encodeObject(self,object):
    return encodeObject(object)

  def joinGroup(self,serviceGroup):
    self._Agent.joinGroup(serviceGroup)

  def leaveGroup(self,serviceGroup):
    self._Agent.leaveGroup(serviceGroup)

  def sendRequest(self,agent,methodName,content):
    return self._Agent.sendRequest(agent,
        methodName,content,self.encodeObject)

  def currentRequest(self):
    return self._currentRequest

  def currentResponse(self):
    return self._currentResponse

  def currentFailure(self):
    return self._currentFailure

  def failureOccurred(self):
    return self._currentFailure != None

  def currentReport(self):
    return self._currentReport

  def currentTimeout(self):
    return self._currentTimeout

  def timeoutOccurred(self):
    return self._currentTimeout != None

  def lastRequest(self):
    return self._lastRequest

  def abortResponse(self,error,description,origin="",details=""):
    raise ServiceFailure(error,description,origin,details)

  def delayResponse(self):
    raise DelayedResponse()

  def executeMethod(self,methodName,method,params):
    return apply(method,params)

  def _localRequest(self,methodName,params):
    method = None
    if self._exportedMethods.has_key(methodName):
      method = self._exportedMethods[methodName]

    try:
      if method != None:
	argcount = method.im_func.func_code.co_argcount-1
	if method.im_func.func_defaults != None:
	  argdflts = len(method.im_func.func_defaults)
	else:
	  argdflts = 0
	varargs = method.im_func.func_code.co_flags & 4
	minargs = argcount - argdflts
	maxargs = argcount
	numargs = len(params)
	if numargs < minargs or (numargs > maxargs and not varargs):
	  error = SERVER_PARAMETERS_INCORRECT
	  description = SERVER_PARAMETERS_INCORRECT_MESSAGE
	  origin = "netsvc"
	  if not varargs:
	    if minargs != maxargs:
	      details = "Expected %d-%d arguments, received %d."
	      details = details % (minargs,maxargs,numargs)
	    else:
	      details = "Expected %d arguments, received %d."
	      details = details % (minargs,numargs)
	  else:
	    details = "Expected %d or more arguments, received %d."
	    details = details % (minargs,numargs)
	  self.abortResponse(error,description,origin,details)
	result = self.executeMethod(methodName,method,params)
      else:
	result = self.invalidMethod(methodName,params)
    except ServiceFailure,exception:
      if exception.origin == "netsvc":
        exception.origin = "netsvc"
      raise exception
    except SystemExit:
      raise
    except:
      error = SERVER_APPLICATION_ERROR
      description = SERVER_APPLICATION_ERROR_MESSAGE
      origin = "netsvc"
      details = exceptionDetails()
      self.abortResponse(error,description,origin,details)
    else:
      return result

  def _handleRequest(self,request):
    try:
      self.handleRequest(_Request(request,self))
    except SystemExit:
      terminate()
    except:
      logException()
      terminate()

  def handleRequest(self,request):
    method = None
    methodName = request.methodName()

    if self._exportedMethods.has_key(methodName):
      method = self._exportedMethods[methodName]

    try:
      params = request.content()
    except:
      error = SERVER_REQUEST_DECODING_ERROR
      description = SERVER_REQUEST_DECODING_ERROR_MESSAGE
      origin = "netsvc"
      details = exceptionDetails()
      request.sendFailure(error,description,origin,details)
      return
    self._currentRequest = request
    try:
      if method != None:
	argcount = method.im_func.func_code.co_argcount-1
	if method.im_func.func_defaults != None:
	  argdflts = len(method.im_func.func_defaults)
	else:
	  argdflts = 0
	varargs = method.im_func.func_code.co_flags & 4
	minargs = argcount - argdflts
	maxargs = argcount
	numargs = len(params)
	if numargs < minargs or (numargs > maxargs and not varargs):
	  error = SERVER_PARAMETERS_INCORRECT
	  description = SERVER_PARAMETERS_INCORRECT_MESSAGE
	  origin = "netsvc"
	  if not varargs:
	    if minargs != maxargs:
	      details = "Expected %d-%d arguments, received %d."
	      details = details % (minargs,maxargs,numargs)
	    else:
	      details = "Expected %d arguments, received %d."
	      details = details % (minargs,numargs)
	  else:
	    details = "Expected %d or more arguments, received %d."
	    details = details % (minargs,numargs)
	  self.abortResponse(error,description,origin,details)
	result = self.executeMethod(methodName,method,params)
      else:
        result = self.invalidMethod(methodName,params)
    except DelayedResponse:
      pass
    except ServiceFailure,exception:
      error = exception.error
      description = exception.description
      origin = exception.origin
      details = exception.details
      request.sendFailure(error,description,origin,details)
    except SystemExit:
      raise
    except:
      error = SERVER_APPLICATION_ERROR
      description = SERVER_APPLICATION_ERROR_MESSAGE
      origin = "netsvc"
      details = exceptionDetails()
      request.sendFailure(error,description,origin,details)
    else:
      try:
	request.sendResponse(result)
      except:
	error = SERVER_RESPONSE_ENCODING_ERROR
	description = SERVER_RESPONSE_ENCODING_ERROR_MESSAGE
	origin = "netsvc"
	details = exceptionDetails()
	request.sendFailure(error,description,origin,details)
    self._currentRequest = None

  def invalidMethod(self,methodName,params):
    error = SERVER_METHOD_UNAVAILABLE
    description = SERVER_METHOD_UNAVAILABLE_MESSAGE
    origin = "netsvc"
    self.abortResponse(error,description,origin)

  def monitorResponse(self,callback,id,timeout=None):
    if id != None:
      if self._lastRequest == None or self._lastRequest[0] != id:
        raise RuntimeError("Doesn't match last conversation.")
      id,binding,methodName = self._lastRequest
      self._activeRequests[id] = (callback,binding,methodName)
      if timeout != None:
	self.startTimer(self._handleResponseTimeout,timeout,id)

  def ignoreResponse(self,id):
    if self._activeRequests.has_key(id):
      del self._activeRequests[id]
      self.cancelTimer(id)

  def _handleResponse(self,response):
    try:
      self.handleResponse(_Response(response,self))
    except SystemExit:
      terminate()
    except:
      logException()
      terminate()

  def handleResponse(self,response):
    id = response.conversationId()
    if self._activeRequests.has_key(id):
      callback,binding,methodName = self._activeRequests[id]
      self.cancelTimer(id)
      del self._activeRequests[id]
      try:
	content = response.content()
      except:
	error = CLIENT_RESPONSE_DECODING_ERROR
	description = CLIENT_RESPONSE_DECODING_ERROR_MESSAGE
	origin = "netsvc"
	details = exceptionDetails()
	self._currentFailure = _LocalFailure(response,
	    error,description,origin,details)
	callback(None)
	self._currentFailure = None
      else:
	self._currentResponse = response
        try:
	  callback(content)
	finally:
	  self._currentResponse = None

  def _handleFailure(self,failure):
    try:
      self.handleFailure(_RemoteFailure(failure))
    except SystemExit:
      terminate()
    except:
      logException()
      terminate()

  def handleFailure(self,failure):
    id = failure.conversationId()
    if self._activeRequests.has_key(id):
      callback,binding,methodName = self._activeRequests[id]
      del self._activeRequests[id]
      self._currentFailure = failure
      try:
	callback(None)
      finally:
        self._currentFailure = None

  def _handleResponseTimeout(self,id):
    if self._activeRequests.has_key(id):
      callback,binding,methodName = self._activeRequests[id]
      error = CLIENT_REQUEST_TIMEOUT
      description = CLIENT_REQUEST_TIMEOUT_MESSAGE
      origin = "netsvc"
      failure = _RequestTimeout(binding,id,0,error,description,origin)
      try:
	self.handleFailure(failure)
	self.ignoreResponse(id)
      except SystemExit:
	terminate()
      except:
	logException()
	terminate()

  def _handleSubscription(self,subscription):
    try:
      self.handleSubscription(subscription)
    except SystemExit:
      terminate()
    except:
      logException()
      terminate()

  def handleSubscription(self,subscription):
    return

  def matchSubject(self,subjectPattern,subjectName):
    return self._Agent.matchSubject(subjectPattern,subjectName)

  def subscribeReports(self,serviceName,subjectPattern):
    self._Agent.subscribeReports(serviceName,subjectPattern)

  def unsubscribeReports(self,serviceName,subjectPattern):
    self._Agent.unsubscribeReports(serviceName,subjectPattern)

  def publishReport(self,subjectName,content,lifeTime=0):
    self._Agent.publishReport(subjectName,
        content,lifeTime,self.encodeObject)

  def sendReport(self,agent,subjectName,content):
    return self._Agent.sendReport(agent,
        subjectName,content,self.encodeObject)

  def _handlePublisherNotification(self,notification):
    try:
      self.handlePublisherNotification(notification)
    except SystemExit:
      terminate()
    except:
      logException()
      terminate()

  def handlePublisherNotification(self,notification):
    return

  def monitorReports(self,callback,service,subjectPattern):
    if not self._activeSubscriptions.has_key(service):
      self._activeSubscriptions[service] = {}
    if not self._activeSubscriptions[service].has_key(subjectPattern):
      self._activeSubscriptions[service][subjectPattern] = callback
      self.subscribeReports(service,subjectPattern)

  def acceptReports(self,callback,service,subjectName):
    if not self._passiveSubscriptions.has_key(service):
      self._passiveSubscriptions[service] = {}
    if not self._passiveSubscriptions[service].has_key(subjectName):
      self._passiveSubscriptions[service][subjectName] = callback

  def ignoreReports(self,service,subjectPattern=None):
    if self._activeSubscriptions.has_key(service):
      if subjectPattern != None:
	if self._activeSubscriptions[service].has_key(subjectPattern):
	  del self._activeSubscriptions[service][subjectPattern]
	  self.unsubscribeReports(service,subjectPattern)
      else:
	for subjectPattern in self._activeSubscriptions[service].keys():
	  self.subscribeReports(service,subjectPattern)
	del self._activeSubscriptions[service]
    if self._passiveSubscriptions.has_key(service):
      if subjectPattern != None:
	if self._passiveSubscriptions[service].has_key(subjectPattern):
	  del self._passiveSubscriptions[service][subjectPattern]
      else:
	del self._passiveSubscriptions[service]

  def _handleReport(self,report):
    try:
      self.handleReport(_Report(report,self))
    except SystemExit:
      terminate()
    except:
      logException()
      terminate()

  def handleReport(self,report):
    service = report.publisher().serviceName()
    subjectName = report.subjectName()
    subjectPattern = report.subjectPattern()
    callback = None
    if self._activeSubscriptions.has_key(service):
      if self._activeSubscriptions[service].has_key(subjectPattern):
        callback = self._activeSubscriptions[service][subjectPattern]
    if callback == None:
      if self._passiveSubscriptions.has_key(service):
	if self._passiveSubscriptions[service].has_key(subjectName):
	  callback = self._passiveSubscriptions[service][subjectName]
    if callback == None:
      callback = self.unexpectedReport
    try:
      content = report.content()
    except SystemExit:
      terminate()
    except:
      service = report.publisher().serviceName()
      subjectName = report.subjectName()
      _logger.notify(LOG_ERROR,"Invalid Report")
      _logger.notify(LOG_ERROR,"Service: %s" % `service`)
      _logger.notify(LOG_ERROR,"SubjectName: %s" % `subjectName`)
      logException()
    else:
      try:
	self._currentReport = report
	callback(service,subjectName,content)
	self._currentReport = None
      except SystemExit:
	self._currentReport = None
	terminate()
      except:
	self._currentReport = None
	logException()
	terminate()

  def unexpectedReport(self,service,subjectName,content):
    return

  def expireReports(self,subjectPattern,age=0):
    self._Agent.expireReports(subjectPattern,age)

  def expireAllReports(self,age):
    self._Agent.expireAllReports(age)

  def purgeReports(self):
    self._Agent.purgeReports()


class _LocalMethod:

  def __init__(self,serviceName,methodName):
    self._serviceName = serviceName
    self._methodName = methodName

  def __getattr__(self,methodName):
    return _LocalMethod(self._serviceName,
        "%s.%s"%(self._methodName,methodName))

  def __call__(self,*args):
    if not Service._services.has_key(self._serviceName):
      error = CLIENT_SERVER_UNAVAILABLE
      description = CLIENT_SERVER_UNAVAILABLE_MESSAGE
      origin = "netsvc"
      details = self._serviceName
      raise ServiceUnavailable(error,description,origin,details)
    service = Service._services[self._serviceName][0]
    return service._localRequest(self._methodName,args)


class LocalService:

  def __init__(self,serviceName):
    self._serviceName = serviceName

  def __getattr__(self,methodName):
    return _LocalMethod(self._serviceName,methodName)

  def __coerce__(self,other):
    if other == None:
      if not Service._services.has_key(self._serviceName):
	return (None,other)
    return None

  def __nonzero__(self):
    return Service._services.has_key(self._serviceName)


# Setup constants for exchange.

EXCHANGE_CLIENT = 0
EXCHANGE_SERVER = 1

CONNECTION_ACTIVE = 0
CONNECTION_INACTIVE = 1

# Exchange wrapper.

class Exchange:

  def __init__(self,mode,group=""):
    self._Exchange = _ose.new_Exchange(self,mode,group)

  def connect(self,hostname,port,retryDelay=-1):
    return self._Exchange.connect(hostname,port,retryDelay)

  def listen(self,port):
    self._Exchange.listen(port)

  def _authorise(self,host):
    try:
      result = self.authorise(host)
      if result:
        return not None
      else:
        return None
    except SystemExit:
      terminate()
    except:
      logException()
      terminate()

  def authorise(self,host):
    return not None

  def _handleConnection(self,announcement):
    try:
      self.handleConnection(announcement)
    except SystemExit:
      terminate()
    except:
      logException()
      terminate()

  def handleConnection(self,announcement):
    return


# HTTP daemon and server wrappers.

CONNECTION_BLOCKED = 0
CONNECTION_CLEARED = 1
CONNECTION_BACKLOG = 2

class HttpDaemon:

  def __init__(self,port):
    self._HttpDaemon = _ose.new_HttpDaemon(self,port)

  def port(self):
    return self._HttpDaemon.port()

  def _authorise(self,host):
    try:
      result = self.authorise(host)
      if result:
        return not None
      return None
    except SystemExit:
      terminate()
    except:
      logException()
      terminate()

  def authorise(self,host):
    return not None

  def start(self):
    self._HttpDaemon.start()

  def stop(self):
    self._HttpDaemon.stop()

  def attach(self,path,server):
    if server != None:
      self._HttpDaemon.attach(path,server._HttpServer.instance())
    else:
      self._HttpDaemon.attach(path,None)


class HttpServlet:

  _bufferedMaximum = 4096

  def __init__(self,session):
    self.serverRoot = session.serverRoot
    self.servletPath = session.servletPath
    self.servletInfo = session.servletInfo
    self._HttpServlet = _ose.new_HttpServlet(self,session.instance())
    self.contentLength = self._HttpServlet.contentLength
    self.contentType = self._HttpServlet.contentType
    self.requestMethod = self._HttpServlet.requestMethod
    self.url = self._HttpServlet.url
    self.protocolVersion = self._HttpServlet.protocolVersion
    self.serverHost = self._HttpServlet.serverHost
    self.serverPort = self._HttpServlet.serverPort
    self.requestPath = self._HttpServlet.requestPath
    self.queryString = self._HttpServlet.queryString
    self.clientHost = self._HttpServlet.clientHost
    self.clientPort = self._HttpServlet.clientPort
    self.remoteUser = self._HttpServlet.remoteUser
    self.headers = self._HttpServlet.headers
    self.containsHeader = self._HttpServlet.containsHeader
    self.header = self._HttpServlet.header
    self.serverRoot = self._HttpServlet.serverRoot
    self.serverPath = self._HttpServlet.serverPath
    self.servletPath = self._HttpServlet.servletPath
    self.servletInfo = self._HttpServlet.servletInfo
    self.sessionId = self._HttpServlet.sessionId
    self.sessionContext = self._HttpServlet.sessionContext
    self.sendResponse = self._HttpServlet.sendResponse
    self.sendHeader = self._HttpServlet.sendHeader
    self.sendError = self._HttpServlet.sendError
    self.sendComplete = self._HttpServlet.sendComplete
    self.shutdown = self._HttpServlet.shutdown
    self.statusText = self._HttpServlet.statusText
    self.subscribeCongestion = self._HttpServlet.subscribeCongestion
    self.unsubscribeCongestion = self._HttpServlet.unsubscribeCongestion
    self._congestionCallback = None
    self._bufferedOutput = []
    self._bufferedLength = 0
    self._bufferedMaximum = HttpServlet._bufferedMaximum

  def _destroyServlet(self):
    try:
      self.destroyServlet()
    except SystemExit:
      terminate()
    except:
      logException()
      terminate()

  def destroyServlet(self):
    self._congestionCallback = None
    self.unsubscribeCongestion()

  def monitorCongestion(self,callback):
    if self._congestionCallback == None:
      self.subscribeCongestion()
    self._congestionCallback = callback

  def ignoreCongestion(self):
    self._congestionCallback = None
    self.unsubscribeCongestion()

  def _handleCongestion(self,status,pending,threshold):
    try:
      self.handleCongestion(status,pending,threshold)
    except SystemExit:
      terminate()
    except:
      logException()
      terminate()

  def handleCongestion(self,status,pending,threshold):
    if self._congestionCallback != None:
      self._congestionCallback(status,pending,threshold)

  def _processRequest(self):
    try:
      self.processRequest()
    except SystemExit:
      terminate()
    except:
      logException()
      self.shutdown()

  def processRequest(self):
    self.sendError(501)

  def _processContent(self,content):
    try:
      self.processContent(content)
    except SystemExit:
      terminate()
    except:
      logException()
      self.shutdown()

  def processContent(self,content):
    pass

  def endHeaders(self):
    self._HttpServlet.endHeaders()
    self._bufferedOutput = []
    self._bufferedLength = 0

  def sendContent(self,content):
    self._bufferedOutput.append(content)
    self._bufferedLength = self._bufferedLength + len(content)
    if self._bufferedLength > self._bufferedMaximum:
      self.flushContent()

  def flushContent(self):
    if self._bufferedLength != 0:
      self._bufferedOutput = string.join(self._bufferedOutput,"")
      self._HttpServlet.sendContent(self._bufferedOutput)
      self._bufferedOutput = []
      self._bufferedLength = 0

  def endContent(self):
    self.flushContent()
    self._HttpServlet.endContent()


class HttpServer:

  def __init__(self,inner=None):
    if inner == None:
      inner =  _ose.new_HttpServer(self)
    self._HttpServer = inner

  def _authorise(self,login,password):
    try:
      result = self.authorise(login,password)
      if result:
        return not None
      return None
    except SystemExit:
      terminate()
    except:
      logException()
      terminate()

  def authorise(self,login,password):
    return not None

  def _servlet(self,session):
    try:
      result = self.servlet(session)
      if result == None:
        return None
      return result._HttpServlet.instance()
    except SystemExit:
      terminate()
    except:
      logException()
      result = ErrorServlet(session,500)
      return result._HttpServlet.instance()

  def servlet(self,session):
    return None


class ErrorServlet(HttpServlet):

  def __init__(self,session,error,content=None):
    HttpServlet.__init__(self,session)
    self._error = error
    self._content = content

  def processRequest(self):
    self.sendError(self._error,self._content)


class EchoServlet:

  def __init__(self,session):
    self._HttpServlet = _ose.new_EchoServlet(self,session.instance())

  def _destroyServlet(self):
    self._HttpServlet = None


REDIRECT_PERMANENT = 301
REDIRECT_TEMPORARY = 302

class RedirectServlet(HttpServlet):

  def __init__(self,session,location,error=REDIRECT_TEMPORARY):
    HttpServlet.__init__(self,session)
    if location[0] == "/":
      if session.serverPort() != "80":
        location = "http://%s:%s%s" % (session.serverHost(),
	    session.serverPort(),location)
      else:
        location = "http://%s%s" % (session.serverHost(),
	    location)
    self._location = location
    self._error = error

  def processRequest(self):
    self.sendResponse(self._error)
    self.sendHeader("Location",self._location)
    self.endHeaders()
    self.endContent()


class RedirectServer(HttpServer):

  def __init__(self,location,error=REDIRECT_PERMANENT):
    HttpServer.__init__(self)
    self._location = location
    self._error = error

  def servlet(self,session):
    location = string.split(session.url(),'?')
    if session.servletPath() != "":
      location[0] = posixpath.join(self._location,session.servletPath())
    else:
      location[0] = self._location
    location = string.join(location,'?')
    if location[0] == '/':
      if session.serverPort() != "80":
        location = "http://%s:%s%s" % (session.serverHost(),
            session.serverPort(),location)
      else:
        location = "http://%s%s" % (session.serverHost(),location)
    return RedirectServlet(session,location,self._error)


class FileServlet:

  def __init__(self,session,fileName,fileType):
    self._HttpServlet = _ose.new_FileServlet(self,
        session.instance(),fileName,fileType)

  def _destroyServlet(self):
    self._HttpServlet = None


class FileServer(HttpServer):

  def __init__(self,directory):
    HttpServer.__init__(self)
    self._directory = directory
    self._mimeTypes = {}
    self._plugins = {}
    self._views = {}
    self._hidden = {}
    self._index = []

  def _mimeType(self,extension):
    if self._mimeTypes.has_key(extension):
      return self._mimeTypes[extension]
    import mimetypes
    mimeType = mimetypes.guess_type(extension)
    if mimeType != None:
      return mimeType[0]

  def directory(self):
    return self._directory

  def map(self,extension,mimeType):
    self._mimeTypes[extension] = mimeType

  def plugin(self,extension,factory,alias=None):
    if alias != None and alias != extension:
      partial,suffix = posixpath.splitext(alias)
      if not self._views.has_key(suffix):
	self._views[suffix] = []
      self._views[suffix].insert(0,(partial,extension,factory))
    else:
      self._plugins[extension] = factory

  def hide(self,extension):
    self._hidden[extension] = 1

  def index(self,index):
    self._index.append(index)

  def servlet(self,session):
    instance = self._search(session,session.servletPath())
    if instance != None:
      return instance
    return ErrorServlet(session,404)

  def _search(self,session,servletPath,servletInfo=""):
    path = posixpath.join(self._directory,servletPath)
    # handle directory request
    if os.path.isdir(path):
      # directory path must be exact
      if servletInfo != "":
        return None
      # directory needs a trailing slash
      if path[-1:] != '/':
	location = string.split(session.url(),'?')
	location[0] = location[0] + '/'
	location = string.join(location,'?')
	return RedirectServlet(session,location)
      else:
        # directory diverted to index file
        for file in self._index:
	  index = path + file
	  if not os.path.isdir(index):
	    instance = self._search(session,servletPath+file,servletInfo)
	    if instance != None:
	      return instance
	  else:
	    return ErrorServlet(session,403)
    # handle file request
    if os.path.exists(path):
      extension = posixpath.splitext(path)[1]
      # file type cannot be hidden
      if not self._hidden.has_key(extension):
	# file type managed by a plugin
	if self._plugins.has_key(extension):
	  factory = self._plugins[extension]
	  session.bindServlet(servletPath,servletInfo)
	  return factory(session,path)
	# file path must be exact
	if servletInfo != "":
	  return None
	mimeType = self._mimeType(extension)
	return FileServlet(session,path,mimeType)
    # look for plugin through a view
    servletStub,suffix = posixpath.splitext(servletPath)
    if self._views.has_key(suffix):
      for partial,extension,factory in self._views[suffix]:
	actualStub = servletStub
	stub = posixpath.basename(servletStub)
	if len(partial) != 0:
	  if stub[-len(partial):] == partial:
	    actualStub = servletStub[:-len(partial)]
	  else:
	    continue
	path = "%s%s" % (posixpath.join(self._directory,actualStub),extension)
	if os.path.exists(path) and os.path.isfile(path):
	  if factory == None:
	    if self._plugins.has_key(extension):
	      if self._hidden.has_key(suffix):
		continue
	      factory = self._plugins[extension]
          if factory == None:
	    continue
	  session.bindServlet(servletPath,servletInfo)
	  return factory(session,path)
    # consider request has extra path info
    if servletPath != "":
      split = posixpath.split(servletPath)
      if servletInfo != "":
        servletInfo = posixpath.join(split[1],servletInfo)
      else:
        servletInfo = split[1]
      servletPath = split[0]
      if servletPath != "":
	return self._search(session,servletPath,servletInfo)
    return None


class _ModuleCache:

  def __init__(self):
    self._cache = {}
    self._children = {}

  def importModule(self,name,path):
    try:
      fp = None
      fp,file,info = imp.find_module(name,[path])
      name = self._moduleName(file)
      if self._requiresUpdate(name,file):
	module = imp.load_module(name,fp,file,info)
	self._cache[name][0] = module
	children = []
	for object in module.__dict__.values():
	  if type(object) == types.ModuleType:
	    if object.__name__[:14] == "_netsvc_cache_":
	      children.append(object.__name__)
	self._children[name] = children
      else:
	module = self._cache[name][0]
    finally:
      if fp != None:
        fp.close()
    return module

  def _requiresUpdate(self,name,file):
    mtime = os.path.getmtime(file)
    if not self._cache.has_key(name):
      self._cache[name] = [None,time.time(),file]
      return not None
    otime = self._cache[name][1]
    if mtime > otime:
      self._cache[name][1] = time.time()
      return not None
    # Note that the following scheme
    # may not always work in all cases
    # and needs to be reworked a bit.
    # Possibly need to track parent
    # links and invalidate parents when
    # change to children are noticed.
    # Hard to work around problem of
    # local time vs file system time
    # if files are on a remote machine.
    if self._children.has_key(name):
      dependencies = []
      visited = { name: 1 }
      dependencies.extend(self._children[name])
      while len(dependencies) != 0:
        next = dependencies.pop()
	if not visited.has_key(next):
	  if self._cache.has_key(next):
	    cache = self._cache[next]
	    mtime = os.path.getmtime(cache[2])
	    if mtime > cache[1] or mtime > otime:
	      self._cache[name][1] = time.time()
	      return not None
	  else:
	    self._cache[name][1] = time.time()
	    return not None
	  dependencies.extend(self._children[next])
	  visited[next] = 1
    return None

  def _moduleName(self,file):
    name = md5.new(file).digest()
    name = base64.encodestring(name)
    name = string.replace(name," ","")
    name = string.replace(name,"=","")
    name = string.replace(name,"/","_2b_")
    name = string.replace(name,"+","_2f_")
    name = string.replace(name,"\r","")
    name = string.replace(name,"\n","")
    name = "_netsvc_cache_" + name
    name = name + "_" + str(len(file))
    return name


_moduleCache = _ModuleCache()

def ModuleCache():
  return _moduleCache


class PythonPlugin:

  def __init__(self):
    self._cache = {}

  def __call__(self,session,file):
    try:
      path = posixpath.dirname(file)
      name = posixpath.splitext(posixpath.basename(file))[0]
      module = _moduleCache.importModule(name,path)
    except:
      logException()
      return ErrorServlet(session,500)
    if not module.__dict__.has_key("__servlet__"):
      return ErrorServlet(session,500)
    return module.__servlet__(session)


class FormServlet(HttpServlet):

  def __init__(self,session):
    HttpServlet.__init__(self,session)
    self._content = []
    self._contentLength = 0
    self._fields = {}
    global cgi,StringIO
    import cgi,StringIO

  def processRequest(self):
    if self.requestMethod() not in ["GET","POST"]:
      self.sendError(501,"Request method type is not supported.")
    elif self.requestMethod() == "POST" and self.contentLength() < 0:
      self.sendError(400,"Content length required for POST.")
    elif self.requestMethod() == "GET" and self.contentLength() > 0:
      self.sendError(400,"Unexpected content length for GET.")
    elif self.requestMethod() == "GET":
      environ = {}
      environ["REQUEST_METHOD"] = self.requestMethod()
      environ["QUERY_STRING"] = self.queryString()
      headers = self.headers()
      headers["content-type"] = "application/x-www-form-urlencoded"
      try:
	form = cgi.FieldStorage(headers=headers,environ=environ,
	    keep_blank_values=1)
	self._handleRequest(form)
      except:
        logException()
        self.shutdown()
    elif self.requestMethod() == "POST" and self.contentLength() == 0:
      environ = {}
      environ["REQUEST_METHOD"] = self.requestMethod()
      fp = StringIO.StringIO("")
      headers = self.headers()
      try:
	form = cgi.FieldStorage(headers=headers,environ=environ,
	    keep_blank_values=1,fp=fp)
	self._handleRequest(form)
      except:
        logException()
        self.shutdown()

  def processContent(self,content):
    self._content.append(content)
    self._contentLength = self._contentLength + len(content)
    if self._contentLength >= self.contentLength():
      environ = {}
      environ["REQUEST_METHOD"] = self.requestMethod()
      self._content = string.join(self._content,"")
      self._content = self._content[:self.contentLength()]
      fp = StringIO.StringIO(self._content)
      headers = self.headers()
      try:
	form = cgi.FieldStorage(headers=headers,environ=environ,
	    keep_blank_values=1,fp=fp)
	self._handleRequest(form)
      except:
        logException()
        self.shutdown()

  def _handleRequest(self,form):
    self._fields = {}
    try:
      keys = form.keys()
    except TypeError:
      keys = []
    for key in keys:
      value = form[key]
      if type(value) is not types.ListType:
	value = value.value
      else:
	value = map(lambda x: x.value, value)
      self._fields[key] = value
    self.handleRequest()

  def containsField(self,name):
    return self._fields.has_key(name)

  def field(self,name):
    return self._fields[name]

  def fields(self):
    return self._fields

  def handleRequest(self):
    self.sendResponse(501)


class RpcServlet:

  def __init__(self,session,binding):
    self._HttpServlet = _ose.new_RpcServlet(self,
        session.instance(),binding)

  def _destroyServlet(self):
    self._HttpServlet = None


class RpcGateway(HttpServer):

  def __init__(self,group):
    inner = _ose.new_RpcGateway(self,group)
    HttpServer.__init__(self,inner)
