#! /usr/bin/env python
#
# Copyright 2001 DUMPLETON SOFTWARE CONSULTING PTY LIMITED

import sys
import signal
import string
import Tkinter
import os
import time
import fnmatch
import socket
import threading
import netsvc

try:
  import Pmw
except:
  print "This program requires the Python Megawidgets (Pmw) module."
  print "Pmw can be obtained free from 'http://pmw.sourceforge.net'."
  sys.exit()

logger = netsvc.Logger()
dispatcher = netsvc.Dispatcher()

class ReportView(netsvc.Service):

  def __init__(self,service,subject):
    #name = "subscriber@%s:%d" % (os.uname()[1],os.getpid())
    #name = "spy@%s:%d" % (os.uname()[1],os.getpid())
    name = "spy@%s" % netsvc.processIdentity()
    netsvc.Service.__init__(self,name,"*")

    self._service = service
    self._subject = subject

    self._root = Pmw.MegaToplevel()
    self._root.withdraw()

    self._root.title("SPYON:SUBJECT - %s - %s" % (service,subject))
    self._root.protocol("WM_DELETE_WINDOW",self._destroyView)

    g1 = Pmw.Group(self._root.interior(),tag_text="Reports")
    g1.pack(side="top",padx=8,pady=8,fill="both",expand="true")

    f1 = Tkinter.Frame(g1.interior())
    f1.pack(side="top")

    f2 = Tkinter.Frame(f1)
    f2.pack(side="left")

    e1 = Pmw.EntryField(f2,labelpos='w',entry_width=25,
        label_text="Name:",validate=None,value=service,
	entry_state="disabled")
    e1.pack(side="top",padx=8,pady=8)

    e2 = Pmw.EntryField(f2,labelpos='w',entry_width=25,
        label_text="Subject:",validate=None,value=subject,
	entry_state="disabled")
    e2.pack(side="top",padx=8,pady=8)

    Pmw.alignlabels((e1,e2))

    self._eventButtons = Pmw.ButtonBox(f1,orient="horizontal")
    self._eventButtons.pack(side="left",padx=8,pady=8)
    self._eventButtons.add("Clear",width=7,command=self._clearLog)

    self._eventView = Pmw.ScrolledText(g1.interior(),text_padx=4,
        text_pady=4,text_wrap="char",text_state="disabled",
	vscrollmode="static")
    self._eventView.pack(side="top",padx=8,pady=8,fill="both",expand="true")
    self._eventView.yview("moveto",1.0)

    self._root.update()
    self._root.deiconify()
    self._root.update_idletasks()

    self.subscribeReports(service,subject)

  def _destroyView(self):
    self._root.destroy()
    self.destroyReferences()

  def _clearLog(self):
    self._eventView.configure(text_state="normal")
    self._eventView.clear()
    self._eventView.configure(text_state="disabled")
    self._eventView.yview("moveto",1.0)

  def _appendLog(self,message):
    p1,p2 = self._eventView.yview()
    self._eventView.configure(text_state="normal")
    self._eventView.insert("end",message)
    self._eventView.configure(text_state="disabled")
    if (p1 == 0.0 and p2 == 0.0) or p2 == 1.0:
      self._eventView.yview("moveto",1.0)

  def handlePublisherNotification(self,notification):
    agent = notification.publisher().agentIdentity()

    self._appendLog("TIMESTAMP = %s\n" % `str(netsvc.DateTime())`)
    self._appendLog("EVENT     = 'ANNOUNCEMENT'\n")
    self._appendLog("AGENT     = %s\n" % `agent`)

    if notification.status() == netsvc.SERVICE_AVAILABLE:
      self._appendLog("STATUS    = 'AVAILABLE'\n")
    else:
      self._appendLog("STATUS    = 'WITHDRAWN'\n")

    self._appendLog("\n")

  def handleReport(self,report):
    agent = report.publisher().agentIdentity()
    subject = report.subjectName()
    content = str(repr(report.content()))

    self._appendLog("TIMESTAMP = %s\n" % `str(netsvc.DateTime())`)
    self._appendLog("EVENT     = 'REPORT'\n")
    self._appendLog("AGENT     = %s\n" % `agent`)
    self._appendLog("SUBJECT   = %s\n" % `subject`)
    self._appendLog("CONTENT   = %s\n" % content)
    self._appendLog("\n")


class ServiceView(netsvc.Monitor):

  def __init__(self,service):
    netsvc.Monitor.__init__(self)

    self._service = service

    self._agents = {}

    self._updateScheduled = 0

    self._root = Pmw.MegaToplevel()
    self._root.withdraw()

    self._root.title("SPYON:SERVICE - " + service)
    self._root.protocol("WM_DELETE_WINDOW",self._destroyView)

    g1 = Pmw.Group(self._root.interior(),tag_text="Service")
    g1.pack(side="top",padx=8,pady=8,fill="both",expand="true")

    f1 = Tkinter.Frame(g1.interior())
    f1.pack(side="top")

    f2 = Tkinter.Frame(f1)
    f2.pack(side="left")

    e1 = Pmw.EntryField(f2,labelpos='w',entry_width=25,
        label_text="Name:",validate=None,value=service,
	entry_state="disabled")
    e1.pack(side="top",padx=8,pady=8)

    self._subjectField = Pmw.EntryField(f2,labelpos='w',entry_width=25,
        label_text="Subject:",validate=None,value="*",
	command=self._subscribeSubject)
    self._subjectField.pack(side="top",padx=8,pady=8)

    Pmw.alignlabels((e1,self._subjectField))

    b1 = Pmw.ButtonBox(f1,orient="vertical")
    b1.pack(side="right",padx=8,pady=8)
    b1.add("Subscribe",command=self._subscribeSubject)

    self._agentView = Pmw.ScrolledListBox(g1.interior(),
        listbox_height=4,dblclickcommand=self._browseAgent)
    self._agentView.pack(side="top",padx=8,pady=8,fill="both",expand="true")

    self._root.update()
    self._root.deiconify()
    self._root.update_idletasks()

    self.subscribeServiceName(self._handleAnnouncements,self._service)

  def _destroyView(self):
    self._root.destroy()
    self.destroyReferences()

  def _browseAgent(self):
    agents = self._agentView.getcurselection()
    if len(agents) != 0:
      print "browse " + agents[0]

  def _subscribeSubject(self):
    child = ReportView(self._service,self._subjectField.get())

  def _handleAnnouncements(self,binding,status):
    update = 0
    agent = binding.agentIdentity()

    if status == netsvc.SERVICE_AVAILABLE:
      if not self._agents.has_key(agent):
	self._agents[agent] = binding
	update = 1
    else:
      if self._agents.has_key(agent):
	del self._agents[agent]
	update = 1

    if update != 0:
      if not self._updateScheduled:
        self._updateScheduled = 1
	self.startTimer(self._updateView,0.25,"0")

  def _updateView(self,tag):
    self._updateScheduled = 0

    agents = self._agents.keys()
    agents.sort()
    self._agentView.setlist(agents)

    self._root.update_idletasks()


class GroupView(netsvc.Monitor):

  def __init__(self,group):
    netsvc.Monitor.__init__(self)

    self._group = group

    self._services = {}
    self._serviceFilter = "*"

    self._updateScheduled = 0

    self._root = Pmw.MegaToplevel()
    self._root.withdraw()

    self._root.title("SPYON:GROUP - " + group)
    self._root.protocol("WM_DELETE_WINDOW",self._destroyView)

    g1 = Pmw.Group(self._root.interior(),tag_text="Services")
    g1.pack(side="top",padx=8,pady=8,fill="both",expand="true")

    f1 = Tkinter.Frame(g1.interior())
    f1.pack(side="top")

    f2 = Tkinter.Frame(f1)
    f2.pack(side="left")

    e1 = Pmw.EntryField(f2,labelpos='w',entry_width=25,
        label_text="Group:",validate=None,value=group,
	entry_state="disabled")
    e1.pack(side="top",padx=8,pady=8)

    self._serviceField = Pmw.EntryField(f2,labelpos='w',entry_width=25,
        label_text="Name:",validate=None,value=self._serviceFilter,
	command=self._applyFilters)
    self._serviceField.pack(side="top",padx=8,pady=8)

    Pmw.alignlabels((e1,self._serviceField))

    b1 = Pmw.ButtonBox(f1,orient="vertical")
    b1.pack(side="left",padx=8,pady=8)
    b1.add("Filter",command=self._applyFilters)
    b1.add("Reset",command=self._resetFilters)

    self._serviceView = Pmw.ScrolledListBox(g1.interior(),
        listbox_height=12,dblclickcommand=self._browseService)
    self._serviceView.pack(side="top",padx=8,pady=8,fill="both",expand="true")

    self._root.update()
    self._root.deiconify()
    self._root.update_idletasks()

    self.subscribeServiceGroup(self._handleAnnouncements,self._group)

  def _destroyView(self):
    self._root.destroy()
    self.destroyReferences()

  def _applyFilters(self):
    self._serviceFilter = self._serviceField.get()
    self._updateScheduled = 1
    self._updateView(0)

  def _resetFilters(self):
    self._serviceField.setentry("*")
    self._applyFilters()

  def _browseService(self):
    services = self._serviceView.getcurselection()
    if len(services) != 0:
      child = ServiceView(services[0])

  def _handleAnnouncements(self,binding,group,status):
    update = 0
    service = binding.serviceName()

    if status == netsvc.SERVICE_AVAILABLE:
      if self._services.has_key(service):
	self._services[service] = self._services[service] + 1
      else:
	self._services[service] = 1
	update = 1
    else:
      if self._services.has_key(service):
	count = self._services[service] - 1
	if count == 0:
	  del self._services[service]
	  update = 1
	else:
	  self._services[service] = count

    if update != 0:
      if not self._updateScheduled:
        self._updateScheduled = 1
	self.startTimer(self._updateView,0.25,"0")

  def _updateView(self,tag):
    self._updateScheduled = 0

    services = []
    for service in self._services.keys():
      if fnmatch.fnmatch(service,self._serviceFilter):
	services.append(service)
    services.sort()
    self._serviceView.setlist(services)

    self._root.update_idletasks()


class MainView(netsvc.Service):

  def __init__(self,root,host,port):
    #name = "spy@%s:%d" % (os.uname()[1],os.getpid())
    name = "spy@%s" % netsvc.processIdentity()
    netsvc.Service.__init__(self,name)

    self._root = root
    self._root.title("SPYON - " + host + ":" + port)
    self._root.protocol("WM_DELETE_WINDOW",self._destroyView)

    self._services = {}
    self._serviceFilter = "*"

    self._groups = {}
    self._groupFilter = "*"

    self._updateScheduled = 0

    self._publisher = ""
    self._subjects = {}

    self._logState = "stopped"
    self._eventLog = ""

    p1 = Pmw.PanedWidget(self._root,orient="horizontal",
        hull_width=900,hull_height=600)
    p1.pack(side="top",padx=8,pady=8,fill="both",expand="true")

    p1.add("left")
    p1.add("right")

    p2 = Pmw.PanedWidget(p1.pane("left"),orient="vertical")
    p2.pack(side="top",padx=4,pady=4,fill="both",expand="true")

    p2.add("top")
    p2.add("bottom")

    g1 = Pmw.Group(p2.pane("top"),tag_text="Services")
    g1.pack(side="top",padx=4,pady=4,fill="both",expand="true")

    f1 = Tkinter.Frame(g1.interior())
    f1.pack(side="top")

    f2 = Tkinter.Frame(f1)
    f2.pack(side="left")

    self._groupField = Pmw.EntryField(f2,labelpos='w',entry_width=25,
        label_text="Group:",validate=None,value=self._groupFilter,
	command=self._applyFilters)
    self._groupField.pack(side="top",padx=8,pady=8)

    self._serviceField = Pmw.EntryField(f2,labelpos='w',entry_width=25,
        label_text="Name:",validate=None,value=self._serviceFilter,
	command=self._applyFilters)
    self._serviceField.pack(side="top",padx=8,pady=8)

    Pmw.alignlabels((self._groupField,self._serviceField))

    b1 = Pmw.ButtonBox(f1,orient="vertical")
    b1.pack(side="left",padx=8,pady=8)
    b1.add("Filter",command=self._applyFilters)
    b1.add("Reset",command=self._resetFilters)

    self._serviceView = Pmw.ScrolledListBox(g1.interior(),
        listbox_height=18,dblclickcommand=self._browseService)
    self._serviceView.pack(side="top",padx=8,pady=8,fill="both",expand="true")

    g2 = Pmw.Group(p2.pane("bottom"),tag_text="Groups")
    g2.pack(side="top",padx=4,pady=4,fill="both",expand="true")

    self._groupView = Pmw.ScrolledListBox(g2.interior(),
        listbox_height=6,dblclickcommand=self._browseGroup)
    self._groupView.pack(side="top",padx=8,pady=8,fill="both",expand="true")

    p3 = Pmw.PanedWidget(p1.pane("right"),orient="vertical")
    p3.pack(side="top",padx=4,pady=4,fill="both",expand="true")

    p3.add("top")
    p3.add("bottom")

    g3 = Pmw.Group(p3.pane("top"),tag_text="Service")
    g3.pack(side="top",padx=4,pady=4,fill="both",expand="true")

    f3 = Tkinter.Frame(g3.interior())
    f3.pack(side="top")

    self._subscribeField = Pmw.EntryField(f3,labelpos='w',entry_width=25,
        label_text="Name:",validate=None,value="",
	command=self._subscribeService)
    self._subscribeField.pack(side="left",padx=8,pady=8)

    b2 = Pmw.ButtonBox(f3,orient="horizontal")
    b2.pack(side="left",padx=8,pady=8)
    b2.add("Subscribe",command=self._subscribeService)

    self._subjectView = Pmw.ScrolledListBox(g3.interior(),listbox_height=12)
    self._subjectView.pack(side="top",padx=8,pady=8,fill="both",expand="true")

    g4 = Pmw.Group(p3.pane("bottom"),tag_text="Events")
    g4.pack(side="top",padx=4,pady=4,fill="both",expand="true")

    f4 = Tkinter.Frame(g4.interior())
    f4.pack(side="top")

    self._eventButtons = Pmw.ButtonBox(f4,orient="horizontal")
    self._eventButtons.pack(side="left",padx=8,pady=8)
    self._eventButtons.add("Start",width=7,command=self._startLog)
    self._eventButtons.add("Suspend",width=7,state="disabled",
        command=self._suspendLog)
    self._eventButtons.add("Clear",width=7,command=self._clearLog)

    self._eventView = Pmw.ScrolledText(g4.interior(),text_padx=4,
        text_pady=4,text_wrap="char",text_state="disabled",
	text_height=10,vscrollmode="static")
    self._eventView.pack(side="top",padx=8,pady=8,fill="both",expand="true")
    self._eventView.yview("moveto",1.0)

    self._root.after(200,self._wakeup)

    self.joinGroup("SPIES")
    self.subscribeRegistry(self._handleAnnouncements)

  def _wakeup(self):
    self._root.after(200,self._wakeup)

  def _destroyView(self):
    self._root.destroy()
    self.destroyReferences()

  def _applyFilters(self):
    self._serviceFilter = self._serviceField.get()
    self._groupFilter = self._groupField.get()
    self._updateScheduled = 1
    self._updateView(0)

  def _resetFilters(self):
    self._serviceField.setentry("*")
    self._groupField.setentry("*")
    self._applyFilters()

  def _browseService(self):
    services = self._serviceView.getcurselection()
    if len(services) != 0:
      child = ServiceView(services[0])

  def _browseGroup(self):
    groups = self._groupView.getcurselection()
    if len(groups) != 0:
      child = GroupView(groups[0])

  def _subscribeService(self):
    if self._publisher != "":
      self._subjectView.clear()
      self._subjects = {}
      self.unsubscribeReports(self._publisher,"*")
      self._appendLog("TIMESTAMP = %s\n" % `str(netsvc.DateTime())`)
      self._appendLog("EVENT     = 'UNSUBSCRIBE'\n")
      self._appendLog("SERVICE   = %s\n" % `self._publisher`)
      self._appendLog("\n")
      self._publisher = self._subscribeField.get()
      if self._publisher != "":
	self._appendLog("TIMESTAMP = %s\n" % `str(netsvc.DateTime())`)
	self._appendLog("EVENT     = 'SUBSCRIBE'\n")
	self._appendLog("SERVICE   = %s\n" % `self._publisher`)
	self._appendLog("\n")
	self.subscribeReports(self._publisher,"*")
    else:
      self._publisher = self._subscribeField.get()
      if self._publisher != "":
	self._appendLog("TIMESTAMP = %s\n" % `str(netsvc.DateTime())`)
	self._appendLog("EVENT     = 'SUBSCRIBE'\n")
	self._appendLog("SERVICE   = %s\n" % `self._publisher`)
	self._appendLog("\n")
	self.subscribeReports(self._publisher,"*")

  def _clearLog(self):
    self._eventView.configure(text_state="normal")
    self._eventView.clear()
    self._eventView.configure(text_state="disabled")
    self._eventView.yview("moveto",1.0)

  def _appendLog(self,message):
    if self._logState == "running":
      p1,p2 = self._eventView.yview()
      self._eventView.configure(text_state="normal")
      self._eventView.insert("end",message)
      self._eventView.configure(text_state="disabled")
      if (p1 == 0.0 and p2 == 0.0) or p2 == 1.0:
	self._eventView.yview("moveto",1.0)
    elif self._logState == "paused":
      self._eventLog = self._eventLog + message

  def _startLog(self):
    if self._logState == "stopped":
      self._logState = "running"
      self._eventButtons.button(0).configure(text="Stop")
      self._eventButtons.button(1).configure(state="normal")
    elif self._logState == "running":
      self._logState = "stopped"
      self._eventButtons.button(0).configure(text="Start")
      self._eventButtons.button(1).configure(state="disabled")
    elif self._logState == "paused":
      self._logState = "stopped"
      self._eventButtons.button(0).configure(text="Start")
      self._eventButtons.button(1).configure(text="Suspend")
      self._eventButtons.button(1).configure(state="disabled")

  def _suspendLog(self):
    if self._logState == "running":
      self._logState = "paused"
      self._eventButtons.button(1).configure(text="Resume")
    elif self._logState == "paused":
      self._logState = "running"
      self._eventButtons.button(1).configure(text="Suspend")
      if self._eventLog != "":
	p1,p2 = self._eventView.yview()
	self._eventView.configure(text_state="normal")
	self._eventView.insert("end",self._eventLog)
	self._eventView.configure(text_state="disabled")
	if (p1 == 0.0 and p2 == 0.0) or p2 == 1.0:
	  self._eventView.yview("moveto",1.0)
	self._eventLog = ""

  def _handleAnnouncements(self,binding,group,status):
    update = 0
    service = binding.serviceName()

    if group == None:
      if status == netsvc.SERVICE_AVAILABLE:
        if self._services.has_key(service):
	  self._services[service] = self._services[service] + 1
	else:
	  self._services[service] = 1
	  update = 1
      else:
        if self._services.has_key(service):
	  count = self._services[service] - 1
	  if count == 0:
	    del self._services[service]
	    update = 1
	  else:
	    self._services[service] = count
    else:
      if status == netsvc.SERVICE_AVAILABLE:
        if self._groups.has_key(group):
	  services = self._groups[group]
	  if services.has_key(service):
	    self._groups[group][service] = services[service] + 1
	  else:
	    self._groups[group][service] = 1
	    update = 1
	else:
	  self._groups[group] = { service: 1 }
	  update = 1
      else:
        if self._groups.has_key(group):
	  services = self._groups[group]
	  if services.has_key(service):
	    count = services[service] - 1
	    if count == 0:
	      del self._groups[group][service]
	      if len(self._groups[group].keys()) == 0:
	        del self._groups[group]
	      update = 1
	    else:
	      self._groups[group][service] = count

    if update != 0:
      if not self._updateScheduled:
        self._updateScheduled = 1
	self.startTimer(self._updateView,0.25,"0")

  def _updateView(self,tag):
    self._updateScheduled = 0

    if self._groupFilter == "*":
      services = []
      for service in self._services.keys():
	if fnmatch.fnmatch(service,self._serviceFilter):
	  services.append(service)
      services.sort()
      self._serviceView.setlist(services)
    else:
      services = []
      for group in self._groups.keys():
	if fnmatch.fnmatch(group,self._groupFilter):
	  for service in self._groups[group].keys():
	    if fnmatch.fnmatch(service,self._serviceFilter):
	      services.append(service)
      services.sort()
      self._serviceView.setlist(services)

    groups = self._groups.keys()
    groups.sort()
    self._groupView.setlist(groups)

    self._root.update_idletasks()

  def handlePublisherNotification(self,notification):
    agent = notification.publisher().agentIdentity()

    self._appendLog("TIMESTAMP = %s\n" % `str(netsvc.DateTime())`)
    self._appendLog("EVENT     = 'ANNOUNCEMENT'\n")
    self._appendLog("SERVICE   = %s\n" % `self._publisher`)
    self._appendLog("AGENT     = %s\n" % `agent`)

    if notification.status() == netsvc.SERVICE_AVAILABLE:
      self._appendLog("STATUS    = 'AVAILABLE'\n")
    else:
      self._appendLog("STATUS    = 'WITHDRAWN'\n")

    self._appendLog("\n")

  def handleReport(self,report):
    agent = report.publisher().agentIdentity()
    subject = report.subjectName()
    content = str(repr(report.content()))

    self._appendLog("TIMESTAMP = %s\n" % `str(netsvc.DateTime())`)
    self._appendLog("EVENT     = 'REPORT'\n")
    self._appendLog("SERVICE   = %s\n" % `self._publisher`)
    self._appendLog("AGENT     = %s\n" % `agent`)
    self._appendLog("SUBJECT   = %s\n" % `subject`)
    self._appendLog("CONTENT   = %s\n" % content)
    self._appendLog("\n")

    view = []
    self._subjects[subject] = content
    subjects = self._subjects.keys()
    subjects.sort()
    for subject in subjects:
      view.append(subject + " = " + self._subjects[subject])

    self._subjectView.setlist(view)


if __name__ == "__main__":

  if len(sys.argv) != 2:
    print "Usage: spyon hostname:port"
    sys.exit()

  dispatcher.monitor(signal.SIGINT)

  group = ""
  try:
    group,address = string.splitfields(sys.argv[1],'@')
  except:
    host,port = string.splitfields(sys.argv[1],':')
  else:
    host,port = string.splitfields(address,':')

  exchange = netsvc.Exchange(netsvc.EXCHANGE_CLIENT,group)
  exchange.connect(host,int(port),5)

  def run():
    dispatcher.run()
    global root
    root.quit()

  thread = threading.Thread(target=run)
  thread.start()

  root = Pmw.initialise()
  root.withdraw()

  main = MainView(root,host,port)

  root.update()
  root.deiconify()
  root.update_idletasks()

  root.mainloop()
  dispatcher.shutdown(0.2)
